From 531be77138aab9e7d06873d8051236e154050ebf Mon Sep 17 00:00:00 2001
From: Jesse Clark <jjclark1982@gmail.com>
Date: Tue, 14 May 2013 22:26:20 +1000
Subject: [PATCH 01/34] Publish a Node.js require() handler for handlebars
 files

---
 lib/handlebars.js | 11 +++++++++++
 1 file changed, 11 insertions(+)

diff --git a/lib/handlebars.js b/lib/handlebars.js
index 7d85c2610..7767b7a83 100644
--- a/lib/handlebars.js
+++ b/lib/handlebars.js
@@ -21,6 +21,17 @@ Handlebars.create = create;
 
 module.exports = Handlebars; // instantiate an instance
 
+// Publish a Node.js require() handler for handlebars files
+if ((typeof(require) !== "undefined") && require.extensions) {
+    var extension = function(module, filename) {
+        var fs = require("fs");
+        var templateString = fs.readFileSync(filename, 'utf8');
+        module.exports = Handlebars.compile(templateString);
+    };
+    require.extensions[".handlebars"] = extension;
+    require.extensions[".hbs"] = extension;
+}
+
 // BEGIN(BROWSER)
 
 // END(BROWSER)

From 229b82bef56961c62538ca30fbd8c120e63a4494 Mon Sep 17 00:00:00 2001
From: Jesse Clark <jjclark1982@gmail.com>
Date: Wed, 15 May 2013 10:38:45 +1000
Subject: [PATCH 02/34] Add unit tests for require() handler

---
 lib/handlebars.js         | 18 +++++++++---------
 spec/example_1.handlebars |  1 +
 spec/example_2.hbs        |  1 +
 spec/qunit_spec.js        | 20 ++++++++++++++++++++
 4 files changed, 31 insertions(+), 9 deletions(-)
 create mode 100644 spec/example_1.handlebars
 create mode 100644 spec/example_2.hbs

diff --git a/lib/handlebars.js b/lib/handlebars.js
index 7767b7a83..f82ec3bad 100644
--- a/lib/handlebars.js
+++ b/lib/handlebars.js
@@ -21,15 +21,15 @@ Handlebars.create = create;
 
 module.exports = Handlebars; // instantiate an instance
 
-// Publish a Node.js require() handler for handlebars files
-if ((typeof(require) !== "undefined") && require.extensions) {
-    var extension = function(module, filename) {
-        var fs = require("fs");
-        var templateString = fs.readFileSync(filename, 'utf8');
-        module.exports = Handlebars.compile(templateString);
-    };
-    require.extensions[".handlebars"] = extension;
-    require.extensions[".hbs"] = extension;
+// Publish a Node.js require() handler for .handlebars and .hbs files
+if (require.extensions) {
+  var extension = function(module, filename) {
+    var fs = require("fs");
+    var templateString = fs.readFileSync(filename, "utf8");
+    module.exports = Handlebars.compile(templateString);
+  };
+  require.extensions[".handlebars"] = extension;
+  require.extensions[".hbs"] = extension;
 }
 
 // BEGIN(BROWSER)
diff --git a/spec/example_1.handlebars b/spec/example_1.handlebars
new file mode 100644
index 000000000..054e96cb8
--- /dev/null
+++ b/spec/example_1.handlebars
@@ -0,0 +1 @@
+{{foo}}
diff --git a/spec/example_2.hbs b/spec/example_2.hbs
new file mode 100644
index 000000000..963eab972
--- /dev/null
+++ b/spec/example_2.hbs
@@ -0,0 +1 @@
+Hello, {{name}}!
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index e2f7d47d0..717aa23c0 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -1475,6 +1475,26 @@ test('GH-375: Unicode line terminators', function() {
   shouldCompileTo('\u2028', {}, '\u2028');
 });
 
+test('Load .handlebars files with require()', function() {
+  var template = require("./example_1");
+  assert.deepEqual(template, require("./example_1.handlebars"));
+
+  var expected = 'foo\n';
+  var result = template({foo: "foo"});
+
+  equal(result, expected);
+});
+
+test('Load .hbs files with require()', function() {
+  var template = require("./example_2");
+  assert.deepEqual(template, require("./example_2.hbs"));
+
+  var expected = 'Hello, World!\n';
+  var result = template({name: "World"});
+
+  equal(result, expected);
+});
+
 suite('Utils');
 
 test('escapeExpression', function() {

From d649b3581e0794fcdc6aab58d84058b1fe5ee7e5 Mon Sep 17 00:00:00 2001
From: Jesse Clark <jjclark1982@gmail.com>
Date: Wed, 15 May 2013 10:50:01 +1000
Subject: [PATCH 03/34] Only run "require" tests when require() is available

---
 spec/qunit_spec.js | 44 ++++++++++++++++++++++++--------------------
 1 file changed, 24 insertions(+), 20 deletions(-)

diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 717aa23c0..fef5166bc 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -1475,26 +1475,6 @@ test('GH-375: Unicode line terminators', function() {
   shouldCompileTo('\u2028', {}, '\u2028');
 });
 
-test('Load .handlebars files with require()', function() {
-  var template = require("./example_1");
-  assert.deepEqual(template, require("./example_1.handlebars"));
-
-  var expected = 'foo\n';
-  var result = template({foo: "foo"});
-
-  equal(result, expected);
-});
-
-test('Load .hbs files with require()', function() {
-  var template = require("./example_2");
-  assert.deepEqual(template, require("./example_2.hbs"));
-
-  var expected = 'Hello, World!\n';
-  var result = template({name: "World"});
-
-  equal(result, expected);
-});
-
 suite('Utils');
 
 test('escapeExpression', function() {
@@ -1522,3 +1502,27 @@ test('isEmpty', function() {
   equal(Handlebars.Utils.isEmpty('foo'), false);
   equal(Handlebars.Utils.isEmpty({bar: 1}), false);
 });
+
+if (typeof(require) !== 'undefined') {
+  suite('Require');
+
+  test('Load .handlebars files with require()', function() {
+    var template = require("./example_1");
+    assert.deepEqual(template, require("./example_1.handlebars"));
+
+    var expected = 'foo\n';
+    var result = template({foo: "foo"});
+
+    equal(result, expected);
+  });
+
+  test('Load .hbs files with require()', function() {
+    var template = require("./example_2");
+    assert.deepEqual(template, require("./example_2.hbs"));
+
+    var expected = 'Hello, World!\n';
+    var result = template({name: "World"});
+
+    equal(result, expected);
+  });
+}

From f5fe6d0bf6d31a2f9eaa1607c56e9ddd63044437 Mon Sep 17 00:00:00 2001
From: James Gorrie <james@bravenewtalent.com>
Date: Fri, 17 May 2013 16:58:34 +0100
Subject: [PATCH 04/34] Allowed for points to be in partial path name

---
 dist/handlebars.js | 2 +-
 spec/qunit_spec.js | 7 +++++++
 src/handlebars.l   | 4 ++--
 3 files changed, 10 insertions(+), 3 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 96d86ea81..93f862dac 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -644,7 +644,7 @@ case 33: return 5;
 break;
 }
 };
-lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@[a-zA-Z]+)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[a-zA-Z0-9_$:\-]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/]+)/,/^(?:$)/];
+lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@[a-zA-Z]+)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[a-zA-Z0-9_$:\-]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
 lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,33],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"par":{"rules":[31,32],"inclusive":false},"INITIAL":{"rules":[0,1,2,33],"inclusive":true}};
 return lexer;})()
 parser.lexer = lexer;
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index fef5166bc..ad690c60f 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -577,6 +577,13 @@ test("Partials with slash paths", function() {
   shouldCompileToWithPartials(string, [hash, {}, {'shared/dude':dude}], true, "Dudes: Jeepers", "Partials can use literal paths");
 });
 
+test("Partials with slash and point paths", function() {
+  var string = "Dudes: {{> shared/dude.thing}}";
+  var dude = "{{name}}";
+  var hash = {name:"Jeepers", another_dude:"Creepers"};
+  shouldCompileToWithPartials(string, [hash, {}, {'shared/dude.thing':dude}], true, "Dudes: Jeepers", "Partials can use literal with points in paths");
+});
+
 test("Multiple partial registration", function() {
   Handlebars.registerPartial({
     'shared/dude': '{{name}}',
diff --git a/src/handlebars.l b/src/handlebars.l
index 8a17a4e28..d12ab28a5 100644
--- a/src/handlebars.l
+++ b/src/handlebars.l
@@ -44,11 +44,11 @@
 <mu>"true"/[}\s]                 { return 'BOOLEAN'; }
 <mu>"false"/[}\s]                { return 'BOOLEAN'; }
 <mu>\-?[0-9]+/[}\s]              { return 'INTEGER'; }
-<mu>[a-zA-Z0-9_$:\-]+/[=}\s\/.]   { return 'ID'; }
+<mu>[a-zA-Z0-9_$:\-]+/[=}\s\/.]  { return 'ID'; }
 <mu>'['[^\]]*']'                 { yytext = yytext.substr(1, yyleng-2); return 'ID'; }
 <mu>.                            { return 'INVALID'; }
 <par>\s+                         { /*ignore whitespace*/ }
-<par>[a-zA-Z0-9_$\-\/]+           { this.popState(); return 'PARTIAL_NAME'; }
+<par>[a-zA-Z0-9_$\-\/\.]+        { this.popState(); return 'PARTIAL_NAME'; }
 
 <INITIAL,mu><<EOF>>              { return 'EOF'; }
 

From 3aef7f604afb975b303518b14487af3691958e44 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Sun, 19 May 2013 11:30:04 -0500
Subject: [PATCH 05/34] Misc readme updates

---
 README.markdown | 8 ++++----
 1 file changed, 4 insertions(+), 4 deletions(-)

diff --git a/README.markdown b/README.markdown
index 1cba64689..d1b7fa68d 100644
--- a/README.markdown
+++ b/README.markdown
@@ -24,7 +24,7 @@ In general, the syntax of Handlebars.js templates is a superset
 of Mustache templates. For basic syntax, check out the [Mustache
 manpage](http://mustache.github.com/mustache.5.html).
 
-Once you have a template, use the Handlebars.compile method to compile
+Once you have a template, use the `Handlebars.compile` method to compile
 the template into a function. The generated function takes a context
 argument, which will be used to render the template.
 
@@ -93,17 +93,17 @@ templates easier and also changes a tiny detail of how partials work.
 Handlebars.js supports an extended expression syntax that we call paths.
 Paths are made up of typical expressions and . characters. Expressions
 allow you to not only display data from the current context, but to
-display data from contexts that are descendents and ancestors of the
+display data from contexts that are descendants and ancestors of the
 current context.
 
-To display data from descendent contexts, use the `.` character. So, for
+To display data from descendant contexts, use the `.` character. So, for
 example, if your data were structured like:
 
 ```js
 var data = {"person": { "name": "Alan" }, company: {"name": "Rad, Inc." } };
 ```
 
-you could display the person's name from the top-level context with the
+You could display the person's name from the top-level context with the
 following expression:
 
 ```

From c95d9d68ea488c2de0bb5dcee0bb6490218e4e8f Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Sun, 19 May 2013 11:30:26 -0500
Subject: [PATCH 06/34] Use context rather than arg in README examples

---
 README.markdown | 12 ++++++------
 1 file changed, 6 insertions(+), 6 deletions(-)

diff --git a/README.markdown b/README.markdown
index d1b7fa68d..a3696d677 100644
--- a/README.markdown
+++ b/README.markdown
@@ -55,12 +55,12 @@ template. Here's an example, which assumes that your objects have a URL
 embedded in them, as well as the text for a link:
 
 ```js
-Handlebars.registerHelper('link_to', function(context) {
-  return "<a href='" + context.url + "'>" + context.body + "</a>";
+Handlebars.registerHelper('link_to', function() {
+  return "<a href='" + this.url + "'>" + this.body + "</a>";
 });
 
 var context = { posts: [{url: "/hello-world", body: "Hello World!"}] };
-var source = "<ul>{{#posts}}<li>{{{link_to this}}}</li>{{/posts}}</ul>"
+var source = "<ul>{{#posts}}<li>{{{link_to}}}</li>{{/posts}}</ul>"
 
 var template = Handlebars.compile(source);
 template(context);
@@ -130,12 +130,12 @@ When calling a helper, you can pass paths or Strings as parameters. For
 instance:
 
 ```js
-Handlebars.registerHelper('link_to', function(title, context) {
-  return "<a href='/posts" + context.url + "'>" + title + "!</a>"
+Handlebars.registerHelper('link_to', function(title) {
+  return "<a href='/posts" + this.url + "'>" + title + "!</a>"
 });
 
 var context = { posts: [{url: "/hello-world", body: "Hello World!"}] };
-var source = '<ul>{{#posts}}<li>{{{link_to "Post" this}}}</li>{{/posts}}</ul>'
+var source = '<ul>{{#posts}}<li>{{{link_to "Post"}}}</li>{{/posts}}</ul>'
 
 var template = Handlebars.compile(source);
 template(context);

From 7fb975071764205d30af87497a8cac29f243be0e Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Sun, 19 May 2013 11:45:20 -0500
Subject: [PATCH 07/34] Update block helper docs in README

Fixes #231, #259
---
 README.markdown | 28 +++++++++++++++++++++-------
 1 file changed, 21 insertions(+), 7 deletions(-)

diff --git a/README.markdown b/README.markdown
index a3696d677..470734237 100644
--- a/README.markdown
+++ b/README.markdown
@@ -72,6 +72,14 @@ template(context);
 // </ul>
 ```
 
+Helpers take precedence over fields defined on the context. To access a field
+that is masked by a helper, a path reference may be used. In the example above
+a field named `link_to` on the `context` object would be referenced using:
+
+```
+{{./link_to}}
+```
+
 Escaping
 --------
 
@@ -130,7 +138,7 @@ When calling a helper, you can pass paths or Strings as parameters. For
 instance:
 
 ```js
-Handlebars.registerHelper('link_to', function(title) {
+Handlebars.registerHelper('link_to', function(title, options) {
   return "<a href='/posts" + this.url + "'>" + title + "!</a>"
 });
 
@@ -177,12 +185,18 @@ template(data);
 // </ul>
 ```
 
-Whenever the block helper is called it is given two parameters, the
-argument that is passed to the helper, or the current context if no
-argument is passed and the compiled contents of the block. Inside of
-the block helper the value of `this` is the current context, wrapped to
-include a method named `__get__` that helps translate paths into values
-within the helpers.
+Whenever the block helper is called it is given one or more parameters,
+any arguments that are passed in the helper in the call and an `options`
+object containing the `fn` function which executes the block's child.
+The block's current context may be accessed through `this`.
+
+Block helpers have the same syntax as mustache sections but should not be
+confused with one another. Sections are akin to an implicit `each` or
+`with` statement depending on the input data and helpers are explicit
+pieces of code that are free to implement whatever behavior they like.
+The [mustache spec](http://mustache.github.io/mustache.5.html)
+defines the exact behavior of sections. In the case of name conflicts,
+helpers are given priority.
 
 ### Partials
 

From 69bcdf36c7c94781203d016fe2f151f47db125c6 Mon Sep 17 00:00:00 2001
From: Blessan Mathew <blessenm@gmail.com>
Date: Tue, 21 May 2013 20:50:46 +0530
Subject: [PATCH 08/34] Fix for issue #517.

Now when compiiled with amd option and a single input argument, we are
checking to see whether the input is a file or folder.
---
 bin/handlebars | 10 +++++++---
 1 file changed, 7 insertions(+), 3 deletions(-)

diff --git a/bin/handlebars b/bin/handlebars
index d605e74db..5b1ba89c9 100755
--- a/bin/handlebars
+++ b/bin/handlebars
@@ -168,10 +168,14 @@ function processTemplate(template, root) {
     if (argv.simple) {
       output.push(handlebars.precompile(data, options) + '\n');
     } else if (argv.partial) {
-      if(argv.amd && argv._.length == 1){ output.push('return '); }
+      if(argv.amd && (argv._.length == 1 && !fs.statSync(argv._[0]).isDirectory())) {
+        output.push('return ');
+      }
       output.push('Handlebars.partials[\'' + template + '\'] = template(' + handlebars.precompile(data, options) + ');\n');
     } else {
-      if(argv.amd && argv._.length == 1){ output.push('return '); }
+      if(argv.amd && (argv._.length == 1 && !fs.statSync(argv._[0]).isDirectory())) {
+        output.push('return ');
+      }
       output.push('templates[\'' + template + '\'] = template(' + handlebars.precompile(data, options) + ');\n');
     }
   }
@@ -184,7 +188,7 @@ argv._.forEach(function(template) {
 // Output the content
 if (!argv.simple) {
   if (argv.amd) {
-    if(argv._.length > 1){
+    if(argv._.length > 1 || (argv._.length == 1 && fs.statSync(argv._[0]).isDirectory())) {
       if(argv.partial){
         output.push('return Handlebars.partials;\n');
       } else {

From f6b6361ad76d3de88e26d3d260669440c488d6fc Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Tue, 21 May 2013 13:03:01 -0500
Subject: [PATCH 09/34] Fix typo

---
 README.markdown | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/README.markdown b/README.markdown
index 470734237..2e9f01dd9 100644
--- a/README.markdown
+++ b/README.markdown
@@ -349,7 +349,7 @@ Handlebars in the Wild
 * [Ember.js](http://www.emberjs.com) makes Handlebars.js the primary way to
   structure your views, also with automatic data binding support.
 * Les Hill (@leshill) wrote a Rails Asset Pipeline gem named
-  handlebars_assets](http://github.com/leshill/handlebars_assets).
+  [handlebars_assets](http://github.com/leshill/handlebars_assets).
 * [Gist about Synchronous and asynchronous loading of external handlebars templates](https://gist.github.com/2287070)
 
 Helping Out

From a97820f54bef4337513557650230b83f25dfc597 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Tue, 21 May 2013 13:03:41 -0500
Subject: [PATCH 10/34] Update testing notes

Fixes #467
---
 README.markdown | 14 +++++++++-----
 1 file changed, 9 insertions(+), 5 deletions(-)

diff --git a/README.markdown b/README.markdown
index 2e9f01dd9..0461e9885 100644
--- a/README.markdown
+++ b/README.markdown
@@ -357,7 +357,6 @@ Helping Out
 To build Handlebars.js you'll need a few things installed.
 
 * Node.js
-* Jison, for building the compiler - `npm install jison`
 * Ruby
 * therubyracer, for running tests - `gem install therubyracer`
 * rspec, for running tests - `gem install rspec`
@@ -367,11 +366,16 @@ and therubyracer if you've got bundler installed.
 
 To build Handlebars.js from scratch, you'll want to run `rake compile`
 in the root of the project. That will build Handlebars and output the
-results to the dist/ folder. To run tests, run `rake spec`. You can also
-run our set of benchmarks with `rake bench`.
+results to the dist/ folder. To run tests, run `rake test`. You can also
+run our set of benchmarks with `rake bench`. Node tests can be run with
+`npm test` or `rake npm_test`. The default rake target will compile and
+run both test suites.
 
-If you notice any problems, please report
-them to the GitHub issue tracker at
+Some environments, notably Windows, have issues running therubyracer. Under these
+envrionments the `rake compile` and `npm test` should be sufficient to test
+most handlebars functionality.
+
+If you notice any problems, please report them to the GitHub issue tracker at
 [http://github.com/wycats/handlebars.js/issues](http://github.com/wycats/handlebars.js/issues).
 Feel free to contact commondream or wycats through GitHub with any other
 questions or feature requests. To submit changes fork the project and

From 533be694b5ceafeda34875e4dd2dfb2bfd149856 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Tue, 21 May 2013 13:12:11 -0500
Subject: [PATCH 11/34] Use better variable name

---
 lib/handlebars/base.js | 12 ++++++------
 1 file changed, 6 insertions(+), 6 deletions(-)

diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index 9f4fdb6eb..0a9b18cd2 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -129,19 +129,19 @@ Handlebars.registerHelper('each', function(context, options) {
   return ret;
 });
 
-Handlebars.registerHelper('if', function(context, options) {
-  var type = toString.call(context);
-  if(type === functionType) { context = context.call(this); }
+Handlebars.registerHelper('if', function(conditional, options) {
+  var type = toString.call(conditional);
+  if(type === functionType) { conditional = conditional.call(this); }
 
-  if(!context || Handlebars.Utils.isEmpty(context)) {
+  if(!conditional || Handlebars.Utils.isEmpty(conditional)) {
     return options.inverse(this);
   } else {
     return options.fn(this);
   }
 });
 
-Handlebars.registerHelper('unless', function(context, options) {
-  return Handlebars.helpers['if'].call(this, context, {fn: options.inverse, inverse: options.fn});
+Handlebars.registerHelper('unless', function(conditional, options) {
+  return Handlebars.helpers['if'].call(this, conditional, {fn: options.inverse, inverse: options.fn});
 });
 
 Handlebars.registerHelper('with', function(context, options) {

From 4d95d58e157aa5ca212a1d1a5871eb88d7ad814e Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Sat, 25 May 2013 12:27:31 -0500
Subject: [PATCH 12/34] Rebuild

---
 dist/handlebars.js         | 12 ++++++------
 dist/handlebars.runtime.js | 12 ++++++------
 2 files changed, 12 insertions(+), 12 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 93f862dac..1f60ae7b8 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -152,19 +152,19 @@ Handlebars.registerHelper('each', function(context, options) {
   return ret;
 });
 
-Handlebars.registerHelper('if', function(context, options) {
-  var type = toString.call(context);
-  if(type === functionType) { context = context.call(this); }
+Handlebars.registerHelper('if', function(conditional, options) {
+  var type = toString.call(conditional);
+  if(type === functionType) { conditional = conditional.call(this); }
 
-  if(!context || Handlebars.Utils.isEmpty(context)) {
+  if(!conditional || Handlebars.Utils.isEmpty(conditional)) {
     return options.inverse(this);
   } else {
     return options.fn(this);
   }
 });
 
-Handlebars.registerHelper('unless', function(context, options) {
-  return Handlebars.helpers['if'].call(this, context, {fn: options.inverse, inverse: options.fn});
+Handlebars.registerHelper('unless', function(conditional, options) {
+  return Handlebars.helpers['if'].call(this, conditional, {fn: options.inverse, inverse: options.fn});
 });
 
 Handlebars.registerHelper('with', function(context, options) {
diff --git a/dist/handlebars.runtime.js b/dist/handlebars.runtime.js
index 7dd40f62e..8744aa77b 100644
--- a/dist/handlebars.runtime.js
+++ b/dist/handlebars.runtime.js
@@ -152,19 +152,19 @@ Handlebars.registerHelper('each', function(context, options) {
   return ret;
 });
 
-Handlebars.registerHelper('if', function(context, options) {
-  var type = toString.call(context);
-  if(type === functionType) { context = context.call(this); }
+Handlebars.registerHelper('if', function(conditional, options) {
+  var type = toString.call(conditional);
+  if(type === functionType) { conditional = conditional.call(this); }
 
-  if(!context || Handlebars.Utils.isEmpty(context)) {
+  if(!conditional || Handlebars.Utils.isEmpty(conditional)) {
     return options.inverse(this);
   } else {
     return options.fn(this);
   }
 });
 
-Handlebars.registerHelper('unless', function(context, options) {
-  return Handlebars.helpers['if'].call(this, context, {fn: options.inverse, inverse: options.fn});
+Handlebars.registerHelper('unless', function(conditional, options) {
+  return Handlebars.helpers['if'].call(this, conditional, {fn: options.inverse, inverse: options.fn});
 });
 
 Handlebars.registerHelper('with', function(context, options) {

From bff71d79d372a8edccfb6b282715634ac32ac3bb Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Sat, 25 May 2013 12:32:16 -0500
Subject: [PATCH 13/34] Update release notes

---
 release-notes.md | 6 +++++-
 1 file changed, 5 insertions(+), 1 deletion(-)

diff --git a/release-notes.md b/release-notes.md
index fb409c686..2badee074 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -1,6 +1,10 @@
 # Release Notes
 
 ## Development
+- [#515](https://github.com/wycats/handlebars.js/issues/515) - Add node require extensions support ([@jjclark1982](https://github.com/jjclark1982))
+- [#517](https://github.com/wycats/handlebars.js/issues/517) - Fix amd precompiler output with directories ([@blessenm](https://github.com/blessenm))
+- [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
+- Docs updates
 
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
 
@@ -26,7 +30,7 @@
 - Fix `toString` handling under IE and browserify ([@tommydudebreaux](https://github.com/tommydudebreaux))
 - Add program metadata
 
-[Commits](https://github.com/wycats/handlebars.js/compare/v1.0.10...master)
+[Commits](https://github.com/wycats/handlebars.js/compare/v1.0.10...v1.0.11)
 
 ## v1.0.10 - Node - Feb 27 2013
 

From c1020a01309723b00ef21525d7f8093efb4d0d93 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 11:52:32 -0500
Subject: [PATCH 14/34] Add unicode support for ID tokens

Fixes #433
Fixes #469
---
 dist/handlebars.js  |  2 +-
 release-notes.md    |  2 ++
 spec/parser_spec.rb |  4 ++++
 src/handlebars.l    | 15 +++++++++++++--
 4 files changed, 20 insertions(+), 3 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 1f60ae7b8..b0ab5ae25 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -644,7 +644,7 @@ case 33: return 5;
 break;
 }
 };
-lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@[a-zA-Z]+)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[a-zA-Z0-9_$:\-]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
+lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@[a-zA-Z]+)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
 lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,33],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"par":{"rules":[31,32],"inclusive":false},"INITIAL":{"rules":[0,1,2,33],"inclusive":true}};
 return lexer;})()
 parser.lexer = lexer;
diff --git a/release-notes.md b/release-notes.md
index 2badee074..93dabc2f3 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -4,6 +4,8 @@
 - [#515](https://github.com/wycats/handlebars.js/issues/515) - Add node require extensions support ([@jjclark1982](https://github.com/jjclark1982))
 - [#517](https://github.com/wycats/handlebars.js/issues/517) - Fix amd precompiler output with directories ([@blessenm](https://github.com/blessenm))
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
+- [#433](https://github.com/wycats/handlebars.js/issues/433) - Add support for unicode ids
+- [#469](https://github.com/wycats/handlebars.js/issues/469) - Add support for `?` in ids
 - Docs updates
 
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
diff --git a/spec/parser_spec.rb b/spec/parser_spec.rb
index 293fe1f74..0b58ce1b3 100644
--- a/spec/parser_spec.rb
+++ b/spec/parser_spec.rb
@@ -125,6 +125,10 @@ def path(*parts)
 
   it "parses simple mustaches" do
     ast_for("{{foo}}").should == root { mustache id("foo") }
+    ast_for("{{foo?}}").should == root { mustache id("foo?") }
+    ast_for("{{foo_}}").should == root { mustache id("foo_") }
+    ast_for("{{foo-}}").should == root { mustache id("foo-") }
+    ast_for("{{foo:}}").should == root { mustache id("foo:") }
   end
 
   it "parses simple mustaches with data" do
diff --git a/src/handlebars.l b/src/handlebars.l
index d12ab28a5..349e78f26 100644
--- a/src/handlebars.l
+++ b/src/handlebars.l
@@ -32,7 +32,7 @@
 <mu>"{{"                         { return 'OPEN'; }
 
 <mu>"="                          { return 'EQUALS'; }
-<mu>"."/[}/ ]                    { return 'ID'; }
+<mu>"."/[}\/ ]                   { return 'ID'; }
 <mu>".."                         { return 'ID'; }
 <mu>[\/.]                        { return 'SEP'; }
 <mu>\s+                          { /*ignore whitespace*/ }
@@ -44,7 +44,18 @@
 <mu>"true"/[}\s]                 { return 'BOOLEAN'; }
 <mu>"false"/[}\s]                { return 'BOOLEAN'; }
 <mu>\-?[0-9]+/[}\s]              { return 'INTEGER'; }
-<mu>[a-zA-Z0-9_$:\-]+/[=}\s\/.]  { return 'ID'; }
+
+/*
+ID is the inverse of control characters.
+Control characters ranges:
+  [\s]          Whitespace
+  [!"#%-,\./]   !, ", #, %, &, ', (, ), *, +, ,, ., /,  Exceptions in range: $, -
+  [;->@]        ;, <, =, >, @,                          Exceptions in range: :, ?
+  [\[-\^`]      [, \, ], ^, `,                          Exceptions in range: _
+  [\{-~]        {, |, }, ~
+*/
+<mu>[^\s!"#%-,\.\/;->@\[-\^`\{-~]+/[=}\s\/.]   { return 'ID'; }
+
 <mu>'['[^\]]*']'                 { yytext = yytext.substr(1, yyleng-2); return 'ID'; }
 <mu>.                            { return 'INVALID'; }
 <par>\s+                         { /*ignore whitespace*/ }

From 5f349913aa1e3efebcd0a6835d33161c7c81d7a9 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 12:29:57 -0500
Subject: [PATCH 15/34] Update in the wild section

Fixes #246
---
 README.markdown | 4 ++++
 1 file changed, 4 insertions(+)

diff --git a/README.markdown b/README.markdown
index 0461e9885..ff1ce7534 100644
--- a/README.markdown
+++ b/README.markdown
@@ -351,6 +351,10 @@ Handlebars in the Wild
 * Les Hill (@leshill) wrote a Rails Asset Pipeline gem named
   [handlebars_assets](http://github.com/leshill/handlebars_assets).
 * [Gist about Synchronous and asynchronous loading of external handlebars templates](https://gist.github.com/2287070)
+* [Lumbar](walmartlabs.github.io/lumbar) provides easy module-based template management for handlebars projects.
+* [YUI](http://yuilibrary.com/yui/docs/handlebars/) implements a port of handlebars
+
+Have a project using Handlebars? Send us a [pull request](https://github.com/wycats/handlebars.js/pull/new/master)!
 
 Helping Out
 -----------

From 822a8911ecd4ffe822fd82afbf7b24704e79f787 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 14:40:52 -0500
Subject: [PATCH 16/34] Add support for complex ids in @data references

---
 dist/handlebars.js                  | 42 +++++++++++++++----------
 lib/handlebars/compiler/compiler.js | 12 ++++++--
 lib/handlebars/compiler/printer.js  |  2 +-
 release-notes.md                    |  1 +
 spec/parser_spec.rb                 |  2 +-
 spec/qunit_spec.js                  | 48 +++++++++++++++++++++++++++++
 spec/tokenizer_spec.rb              | 12 ++++----
 src/handlebars.l                    |  2 +-
 src/handlebars.yy                   | 10 ++++--
 9 files changed, 101 insertions(+), 30 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index b0ab5ae25..cb3d8a9aa 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -181,9 +181,9 @@ Handlebars.registerHelper('log', function(context, options) {
 var handlebars = (function(){
 var parser = {trace: function trace() { },
 yy: {},
-symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"OPEN_PARTIAL":24,"partialName":25,"params":26,"hash":27,"DATA":28,"param":29,"STRING":30,"INTEGER":31,"BOOLEAN":32,"hashSegments":33,"hashSegment":34,"ID":35,"EQUALS":36,"PARTIAL_NAME":37,"pathSegments":38,"SEP":39,"$accept":0,"$end":1},
-terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"OPEN_PARTIAL",28:"DATA",30:"STRING",31:"INTEGER",32:"BOOLEAN",35:"ID",36:"EQUALS",37:"PARTIAL_NAME",39:"SEP"},
-productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[26,2],[26,1],[29,1],[29,1],[29,1],[29,1],[29,1],[27,1],[33,2],[33,1],[34,3],[34,3],[34,3],[34,3],[34,3],[25,1],[21,1],[38,3],[38,1]],
+symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"OPEN_PARTIAL":24,"partialName":25,"params":26,"hash":27,"dataName":28,"param":29,"STRING":30,"INTEGER":31,"BOOLEAN":32,"hashSegments":33,"hashSegment":34,"ID":35,"EQUALS":36,"PARTIAL_NAME":37,"DATA":38,"pathSegments":39,"SEP":40,"$accept":0,"$end":1},
+terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"OPEN_PARTIAL",30:"STRING",31:"INTEGER",32:"BOOLEAN",35:"ID",36:"EQUALS",37:"PARTIAL_NAME",38:"DATA",40:"SEP"},
+productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[26,2],[26,1],[29,1],[29,1],[29,1],[29,1],[29,1],[27,1],[33,2],[33,1],[34,3],[34,3],[34,3],[34,3],[34,3],[25,1],[28,2],[21,1],[39,3],[39,1]],
 performAction: function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$) {
 
 var $0 = $$.length - 1;
@@ -242,7 +242,7 @@ case 26: this.$ = [[$$[$0-1]], $$[$0]];
 break;
 case 27: this.$ = [[$$[$0]], null]; 
 break;
-case 28: this.$ = [[new yy.DataNode($$[$0])], null]; 
+case 28: this.$ = [[$$[$0]], null]; 
 break;
 case 29: $$[$0-1].push($$[$0]); this.$ = $$[$0-1]; 
 break;
@@ -256,7 +256,7 @@ case 33: this.$ = new yy.IntegerNode($$[$0]);
 break;
 case 34: this.$ = new yy.BooleanNode($$[$0]); 
 break;
-case 35: this.$ = new yy.DataNode($$[$0]); 
+case 35: this.$ = $$[$0]; 
 break;
 case 36: this.$ = new yy.HashNode($$[$0]); 
 break;
@@ -272,20 +272,22 @@ case 41: this.$ = [$$[$0-2], new yy.IntegerNode($$[$0])];
 break;
 case 42: this.$ = [$$[$0-2], new yy.BooleanNode($$[$0])]; 
 break;
-case 43: this.$ = [$$[$0-2], new yy.DataNode($$[$0])]; 
+case 43: this.$ = [$$[$0-2], $$[$0]]; 
 break;
 case 44: this.$ = new yy.PartialNameNode($$[$0]); 
 break;
-case 45: this.$ = new yy.IdNode($$[$0]); 
+case 45: this.$ = new yy.DataNode($$[$0]); 
 break;
-case 46: $$[$0-2].push($$[$0]); this.$ = $$[$0-2]; 
+case 46: this.$ = new yy.IdNode($$[$0]); 
 break;
-case 47: this.$ = [$$[$0]]; 
+case 47: $$[$0-2].push($$[$0]); this.$ = $$[$0-2]; 
+break;
+case 48: this.$ = [$$[$0]]; 
 break;
 }
 },
-table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],24:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],24:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],24:[1,16]},{17:23,18:[1,22],21:24,28:[1,25],35:[1,27],38:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],24:[2,8]},{4:28,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],24:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],24:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],24:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],24:[2,15]},{17:30,21:24,28:[1,25],35:[1,27],38:26},{17:31,21:24,28:[1,25],35:[1,27],38:26},{17:32,21:24,28:[1,25],35:[1,27],38:26},{25:33,37:[1,34]},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],24:[1,16]},{17:23,21:24,28:[1,25],35:[1,27],38:26},{5:[2,4],7:35,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],24:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],24:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],24:[2,23]},{18:[1,36]},{18:[2,27],21:41,26:37,27:38,28:[1,45],29:39,30:[1,42],31:[1,43],32:[1,44],33:40,34:46,35:[1,47],38:26},{18:[2,28]},{18:[2,45],28:[2,45],30:[2,45],31:[2,45],32:[2,45],35:[2,45],39:[1,48]},{18:[2,47],28:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],39:[2,47]},{10:49,20:[1,50]},{10:51,20:[1,50]},{18:[1,52]},{18:[1,53]},{18:[1,54]},{18:[1,55],21:56,35:[1,27],38:26},{18:[2,44],35:[2,44]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],24:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],24:[2,17]},{18:[2,25],21:41,27:57,28:[1,45],29:58,30:[1,42],31:[1,43],32:[1,44],33:40,34:46,35:[1,47],38:26},{18:[2,26]},{18:[2,30],28:[2,30],30:[2,30],31:[2,30],32:[2,30],35:[2,30]},{18:[2,36],34:59,35:[1,60]},{18:[2,31],28:[2,31],30:[2,31],31:[2,31],32:[2,31],35:[2,31]},{18:[2,32],28:[2,32],30:[2,32],31:[2,32],32:[2,32],35:[2,32]},{18:[2,33],28:[2,33],30:[2,33],31:[2,33],32:[2,33],35:[2,33]},{18:[2,34],28:[2,34],30:[2,34],31:[2,34],32:[2,34],35:[2,34]},{18:[2,35],28:[2,35],30:[2,35],31:[2,35],32:[2,35],35:[2,35]},{18:[2,38],35:[2,38]},{18:[2,47],28:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],36:[1,61],39:[2,47]},{35:[1,62]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],24:[2,10]},{21:63,35:[1,27],38:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],24:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],24:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],24:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],24:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],24:[2,21]},{18:[1,64]},{18:[2,24]},{18:[2,29],28:[2,29],30:[2,29],31:[2,29],32:[2,29],35:[2,29]},{18:[2,37],35:[2,37]},{36:[1,61]},{21:65,28:[1,69],30:[1,66],31:[1,67],32:[1,68],35:[1,27],38:26},{18:[2,46],28:[2,46],30:[2,46],31:[2,46],32:[2,46],35:[2,46],39:[2,46]},{18:[1,70]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],24:[2,22]},{18:[2,39],35:[2,39]},{18:[2,40],35:[2,40]},{18:[2,41],35:[2,41]},{18:[2,42],35:[2,42]},{18:[2,43],35:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],24:[2,18]}],
-defaultActions: {17:[2,1],25:[2,28],38:[2,26],57:[2,24]},
+table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],24:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],24:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],24:[1,16]},{17:23,18:[1,22],21:24,28:25,35:[1,28],38:[1,27],39:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],24:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],24:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],24:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],24:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],24:[2,15]},{17:31,21:24,28:25,35:[1,28],38:[1,27],39:26},{17:32,21:24,28:25,35:[1,28],38:[1,27],39:26},{17:33,21:24,28:25,35:[1,28],38:[1,27],39:26},{25:34,37:[1,35]},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],24:[1,16]},{17:23,21:24,28:25,35:[1,28],38:[1,27],39:26},{5:[2,4],7:36,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],24:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],24:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],24:[2,23]},{18:[1,37]},{18:[2,27],21:42,26:38,27:39,28:46,29:40,30:[1,43],31:[1,44],32:[1,45],33:41,34:47,35:[1,48],38:[1,27],39:26},{18:[2,28]},{18:[2,46],30:[2,46],31:[2,46],32:[2,46],35:[2,46],38:[2,46],40:[1,49]},{21:50,35:[1,28],39:26},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],38:[2,48],40:[2,48]},{10:51,20:[1,52]},{10:53,20:[1,52]},{18:[1,54]},{18:[1,55]},{18:[1,56]},{18:[1,57],21:58,35:[1,28],39:26},{18:[2,44],35:[2,44]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],24:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],24:[2,17]},{18:[2,25],21:42,27:59,28:46,29:60,30:[1,43],31:[1,44],32:[1,45],33:41,34:47,35:[1,48],38:[1,27],39:26},{18:[2,26]},{18:[2,30],30:[2,30],31:[2,30],32:[2,30],35:[2,30],38:[2,30]},{18:[2,36],34:61,35:[1,62]},{18:[2,31],30:[2,31],31:[2,31],32:[2,31],35:[2,31],38:[2,31]},{18:[2,32],30:[2,32],31:[2,32],32:[2,32],35:[2,32],38:[2,32]},{18:[2,33],30:[2,33],31:[2,33],32:[2,33],35:[2,33],38:[2,33]},{18:[2,34],30:[2,34],31:[2,34],32:[2,34],35:[2,34],38:[2,34]},{18:[2,35],30:[2,35],31:[2,35],32:[2,35],35:[2,35],38:[2,35]},{18:[2,38],35:[2,38]},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],36:[1,63],38:[2,48],40:[2,48]},{35:[1,64]},{18:[2,45],30:[2,45],31:[2,45],32:[2,45],35:[2,45],38:[2,45]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],24:[2,10]},{21:65,35:[1,28],39:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],24:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],24:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],24:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],24:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],24:[2,21]},{18:[1,66]},{18:[2,24]},{18:[2,29],30:[2,29],31:[2,29],32:[2,29],35:[2,29],38:[2,29]},{18:[2,37],35:[2,37]},{36:[1,63]},{21:67,28:71,30:[1,68],31:[1,69],32:[1,70],35:[1,28],38:[1,27],39:26},{18:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],38:[2,47],40:[2,47]},{18:[1,72]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],24:[2,22]},{18:[2,39],35:[2,39]},{18:[2,40],35:[2,40]},{18:[2,41],35:[2,41]},{18:[2,42],35:[2,42]},{18:[2,43],35:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],24:[2,18]}],
+defaultActions: {17:[2,1],25:[2,28],39:[2,26],59:[2,24]},
 parseError: function parseError(str, hash) {
     throw new Error(str);
 },
@@ -610,7 +612,7 @@ case 16: return 35;
 break;
 case 17: return 35; 
 break;
-case 18: return 39; 
+case 18: return 40; 
 break;
 case 19: /*ignore whitespace*/ 
 break;
@@ -622,7 +624,7 @@ case 22: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\"/g,'"'); ret
 break;
 case 23: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\'/g,"'"); return 30; 
 break;
-case 24: yy_.yytext = yy_.yytext.substr(1); return 28; 
+case 24: return 38; 
 break;
 case 25: return 32; 
 break;
@@ -644,7 +646,7 @@ case 33: return 5;
 break;
 }
 };
-lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@[a-zA-Z]+)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
+lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
 lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,33],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"par":{"rules":[31,32],"inclusive":false},"INITIAL":{"rules":[0,1,2,33],"inclusive":true}};
 return lexer;})()
 parser.lexer = lexer;
@@ -1162,7 +1164,15 @@ Compiler.prototype = {
 
   DATA: function(data) {
     this.options.data = true;
-    this.opcode('lookupData', data.id);
+    if (data.id.isScoped || data.id.depth) {
+      throw new Handlebars.Exception('Scoped data references are not supported: ' + data.original);
+    }
+
+    this.opcode('lookupData');
+    var parts = data.id.parts;
+    for(var i=0, l=parts.length; i<l; i++) {
+      this.opcode('lookup', parts[i]);
+    }
   },
 
   STRING: function(string) {
@@ -1610,7 +1620,7 @@ JavaScriptCompiler.prototype = {
   //
   // Push the result of looking up `id` on the current data
   lookupData: function(id) {
-    this.push(this.nameLookup('data', id, 'data'));
+    this.push('data');
   },
 
   // [pushStringParam]
diff --git a/lib/handlebars/compiler/compiler.js b/lib/handlebars/compiler/compiler.js
index 98f5396e4..d40e5f88f 100644
--- a/lib/handlebars/compiler/compiler.js
+++ b/lib/handlebars/compiler/compiler.js
@@ -301,7 +301,15 @@ Compiler.prototype = {
 
   DATA: function(data) {
     this.options.data = true;
-    this.opcode('lookupData', data.id);
+    if (data.id.isScoped || data.id.depth) {
+      throw new Handlebars.Exception('Scoped data references are not supported: ' + data.original);
+    }
+
+    this.opcode('lookupData');
+    var parts = data.id.parts;
+    for(var i=0, l=parts.length; i<l; i++) {
+      this.opcode('lookup', parts[i]);
+    }
   },
 
   STRING: function(string) {
@@ -749,7 +757,7 @@ JavaScriptCompiler.prototype = {
   //
   // Push the result of looking up `id` on the current data
   lookupData: function(id) {
-    this.push(this.nameLookup('data', id, 'data'));
+    this.push('data');
   },
 
   // [pushStringParam]
diff --git a/lib/handlebars/compiler/printer.js b/lib/handlebars/compiler/printer.js
index 780770e01..d9eb7a577 100644
--- a/lib/handlebars/compiler/printer.js
+++ b/lib/handlebars/compiler/printer.js
@@ -121,7 +121,7 @@ Handlebars.PrintVisitor.prototype.PARTIAL_NAME = function(partialName) {
 };
 
 Handlebars.PrintVisitor.prototype.DATA = function(data) {
-  return "@" + data.id;
+  return "@" + this.accept(data.id);
 };
 
 Handlebars.PrintVisitor.prototype.content = function(content) {
diff --git a/release-notes.md b/release-notes.md
index 93dabc2f3..b233bd621 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -6,6 +6,7 @@
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
 - [#433](https://github.com/wycats/handlebars.js/issues/433) - Add support for unicode ids
 - [#469](https://github.com/wycats/handlebars.js/issues/469) - Add support for `?` in ids
+- Add support for complex ids in @data references
 - Docs updates
 
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
diff --git a/spec/parser_spec.rb b/spec/parser_spec.rb
index 0b58ce1b3..e478aa5b8 100644
--- a/spec/parser_spec.rb
+++ b/spec/parser_spec.rb
@@ -111,7 +111,7 @@ def id(id)
     end
 
     def data(id)
-      "@#{id}"
+      "@ID:#{id}"
     end
 
     def partial_name(name)
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index ad690c60f..23dcdb9ae 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -940,6 +940,54 @@ test("hash values can be looked up via @foo", function() {
   equals("Hello world", result, "@foo as a parameter retrieves template data");
 });
 
+test("nested parameter data can be looked up via @foo.bar", function() {
+  var template = CompilerContext.compile("{{hello @world.bar}}");
+  var helpers = {
+    hello: function(noun) {
+      return "Hello " + noun;
+    }
+  };
+
+  var result = template({}, { helpers: helpers, data: { world: {bar: "world" } } });
+  equals("Hello world", result, "@foo as a parameter retrieves template data");
+});
+
+test("nested parameter data does not fail with @world.bar", function() {
+  var template = CompilerContext.compile("{{hello @world.bar}}");
+  var helpers = {
+    hello: function(noun) {
+      return "Hello " + noun;
+    }
+  };
+
+  var result = template({}, { helpers: helpers, data: { foo: {bar: "world" } } });
+  equals("Hello undefined", result, "@foo as a parameter retrieves template data");
+});
+
+test("parameter data throws when using this scope references", function() {
+  var string = "{{#goodbyes}}{{text}} cruel {{@./name}}! {{/goodbyes}}";
+
+  shouldThrow(function() {
+      CompilerContext.compile(string);
+    }, Error, "Should throw exception");
+});
+
+test("parameter data throws when using parent scope references", function() {
+  var string = "{{#goodbyes}}{{text}} cruel {{@../name}}! {{/goodbyes}}";
+
+  shouldThrow(function() {
+      CompilerContext.compile(string);
+    }, Error, "Should throw exception");
+});
+
+test("parameter data throws when using complex scope references", function() {
+  var string = "{{#goodbyes}}{{text}} cruel {{@foo/../name}}! {{/goodbyes}}";
+
+  shouldThrow(function() {
+      CompilerContext.compile(string);
+    }, Error, "Should throw exception");
+});
+
 test("data is inherited downstream", function() {
   var template = CompilerContext.compile("{{#let foo=bar.baz}}{{@foo}}{{/let}}", { data: true });
   var helpers = {
diff --git a/spec/tokenizer_spec.rb b/spec/tokenizer_spec.rb
index 0240e6310..cc4d0fbff 100644
--- a/spec/tokenizer_spec.rb
+++ b/spec/tokenizer_spec.rb
@@ -280,16 +280,16 @@ def tokenize(string)
 
   it "tokenizes special @ identifiers" do
     result = tokenize("{{ @foo }}")
-    result.should match_tokens %w( OPEN DATA CLOSE )
-    result[1].should be_token("DATA", "foo")
+    result.should match_tokens %w( OPEN DATA ID CLOSE )
+    result[2].should be_token("ID", "foo")
 
     result = tokenize("{{ foo @bar }}")
-    result.should match_tokens %w( OPEN ID DATA CLOSE )
-    result[2].should be_token("DATA", "bar")
+    result.should match_tokens %w( OPEN ID DATA ID CLOSE )
+    result[3].should be_token("ID", "bar")
 
     result = tokenize("{{ foo bar=@baz }}")
-    result.should match_tokens %w( OPEN ID ID EQUALS DATA CLOSE )
-    result[4].should be_token("DATA", "baz")
+    result.should match_tokens %w( OPEN ID ID EQUALS DATA ID CLOSE )
+    result[5].should be_token("ID", "baz")
   end
 
   it "does not time out in a mustache with a single } followed by EOF" do
diff --git a/src/handlebars.l b/src/handlebars.l
index 349e78f26..7fcf86e30 100644
--- a/src/handlebars.l
+++ b/src/handlebars.l
@@ -40,7 +40,7 @@
 <mu>"}}"                         { this.popState(); return 'CLOSE'; }
 <mu>'"'("\\"["]|[^"])*'"'        { yytext = yytext.substr(1,yyleng-2).replace(/\\"/g,'"'); return 'STRING'; }
 <mu>"'"("\\"[']|[^'])*"'"        { yytext = yytext.substr(1,yyleng-2).replace(/\\'/g,"'"); return 'STRING'; }
-<mu>"@"[a-zA-Z]+                 { yytext = yytext.substr(1); return 'DATA'; }
+<mu>"@"                          { return 'DATA'; }
 <mu>"true"/[}\s]                 { return 'BOOLEAN'; }
 <mu>"false"/[}\s]                { return 'BOOLEAN'; }
 <mu>\-?[0-9]+/[}\s]              { return 'INTEGER'; }
diff --git a/src/handlebars.yy b/src/handlebars.yy
index 7ab0153ed..7e6321f8b 100644
--- a/src/handlebars.yy
+++ b/src/handlebars.yy
@@ -61,7 +61,7 @@ inMustache
   | path params { $$ = [[$1].concat($2), null]; }
   | path hash { $$ = [[$1], $2]; }
   | path { $$ = [[$1], null]; }
-  | DATA { $$ = [[new yy.DataNode($1)], null]; }
+  | dataName { $$ = [[$1], null]; }
   ;
 
 params
@@ -74,7 +74,7 @@ param
   | STRING { $$ = new yy.StringNode($1); }
   | INTEGER { $$ = new yy.IntegerNode($1); }
   | BOOLEAN { $$ = new yy.BooleanNode($1); }
-  | DATA { $$ = new yy.DataNode($1); }
+  | dataName { $$ = $1; }
   ;
 
 hash
@@ -91,13 +91,17 @@ hashSegment
   | ID EQUALS STRING { $$ = [$1, new yy.StringNode($3)]; }
   | ID EQUALS INTEGER { $$ = [$1, new yy.IntegerNode($3)]; }
   | ID EQUALS BOOLEAN { $$ = [$1, new yy.BooleanNode($3)]; }
-  | ID EQUALS DATA { $$ = [$1, new yy.DataNode($3)]; }
+  | ID EQUALS dataName { $$ = [$1, $3]; }
   ;
 
 partialName
   : PARTIAL_NAME { $$ = new yy.PartialNameNode($1); }
   ;
 
+dataName
+  : DATA path { $$ = new yy.DataNode($2); }
+  ;
+
 path
   : pathSegments { $$ = new yy.IdNode($1); }
   ;

From 3d51088d4de39ffcf1f87a13f02d8873204580ea Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 14:58:49 -0500
Subject: [PATCH 17/34] Fail if jison fails

---
 Rakefile | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/Rakefile b/Rakefile
index ac935ddb2..a6e3e102b 100644
--- a/Rakefile
+++ b/Rakefile
@@ -10,7 +10,7 @@ def compile_parser
 
     sh "rm handlebars.js"
   else
-    puts "Failed to run Jison."
+    fail "Failed to run Jison."
   end
 end
 

From 2c8dd7499771256c65da13f5e3bb4c6efb834106 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 14:59:45 -0500
Subject: [PATCH 18/34] Add unescaped tokenizer tests

---
 spec/tokenizer_spec.rb | 14 ++++++++++++++
 1 file changed, 14 insertions(+)

diff --git a/spec/tokenizer_spec.rb b/spec/tokenizer_spec.rb
index cc4d0fbff..c090b527b 100644
--- a/spec/tokenizer_spec.rb
+++ b/spec/tokenizer_spec.rb
@@ -41,6 +41,20 @@ def tokenize(string)
     result[1].should be_token("ID", "foo")
   end
 
+  it "supports unescaping with &" do
+    result = tokenize("{{&bar}}")
+    result.should match_tokens(%w(OPEN_UNESCAPED ID CLOSE))
+
+    result[1].should be_token("ID", "bar")
+  end
+
+  it "supports unescaping with {{{" do
+    result = tokenize("{{{bar}}}")
+    result.should match_tokens(%w(OPEN_UNESCAPED ID CLOSE))
+
+    result[1].should be_token("ID", "bar")
+  end
+
   it "supports escaping delimiters" do
     result = tokenize("{{foo}} \\{{bar}} {{baz}}")
     result.should match_tokens(%w(OPEN ID CLOSE CONTENT CONTENT OPEN ID CLOSE))

From 15186dc9ddfcce7d83a2284b065ddf0bb2cf4657 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 19:03:16 -0400
Subject: [PATCH 19/34] Add supported environments statement

Fixes #344
---
 README.markdown | 14 ++++++++++++++
 1 file changed, 14 insertions(+)

diff --git a/README.markdown b/README.markdown
index ff1ce7534..9f7a12223 100644
--- a/README.markdown
+++ b/README.markdown
@@ -290,6 +290,20 @@ normal.
 - When all helpers are known in advance the `--knownOnly` argument may be used
   to optimize all block helper references.
 
+Supported Environments
+----------------------
+
+Handlebars has been designed to work in any ECMAScript 3 environment. This includes
+
+- Node.js
+- Chrome
+- Firefox
+- Safari 5+
+- Opera 11+
+- IE 6+
+
+Older versions and other runtimes are likely to work but have not been formally
+tested.
 
 Performance
 -----------

From 503e32208bcc9e9f5d9052b4a7e732bf707be15e Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Mon, 27 May 2013 19:47:17 -0400
Subject: [PATCH 20/34] Improve tracking of original path values

---
 dist/handlebars.js             | 23 ++++++++++++++---------
 lib/handlebars/compiler/ast.js | 19 ++++++++++++-------
 src/handlebars.yy              |  4 ++--
 3 files changed, 28 insertions(+), 18 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index cb3d8a9aa..56b6fea8a 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -280,9 +280,9 @@ case 45: this.$ = new yy.DataNode($$[$0]);
 break;
 case 46: this.$ = new yy.IdNode($$[$0]); 
 break;
-case 47: $$[$0-2].push($$[$0]); this.$ = $$[$0-2]; 
+case 47: $$[$0-2].push({part: $$[$0], separator: $$[$0-1]}); this.$ = $$[$0-2]; 
 break;
-case 48: this.$ = [$$[$0]]; 
+case 48: this.$ = [{part: $$[$0]}]; 
 break;
 }
 },
@@ -733,21 +733,24 @@ Handlebars.AST.HashNode = function(pairs) {
 
 Handlebars.AST.IdNode = function(parts) {
   this.type = "ID";
-  this.original = parts.join(".");
 
-  var dig = [], depth = 0;
+  var original = "",
+      dig = [],
+      depth = 0;
 
   for(var i=0,l=parts.length; i<l; i++) {
-    var part = parts[i];
+    var part = parts[i].part;
+    original += (parts[i].separator || '') + part;
 
     if (part === ".." || part === "." || part === "this") {
-      if (dig.length > 0) { throw new Handlebars.Exception("Invalid path: " + this.original); }
+      if (dig.length > 0) { throw new Handlebars.Exception("Invalid path: " + original); }
       else if (part === "..") { depth++; }
       else { this.isScoped = true; }
     }
     else { dig.push(part); }
   }
 
+  this.original = original;
   this.parts    = dig;
   this.string   = dig.join('.');
   this.depth    = depth;
@@ -771,13 +774,15 @@ Handlebars.AST.DataNode = function(id) {
 
 Handlebars.AST.StringNode = function(string) {
   this.type = "STRING";
-  this.string = string;
-  this.stringModeValue = string;
+  this.original =
+    this.string =
+    this.stringModeValue = string;
 };
 
 Handlebars.AST.IntegerNode = function(integer) {
   this.type = "INTEGER";
-  this.integer = integer;
+  this.original =
+    this.integer = integer;
   this.stringModeValue = Number(integer);
 };
 
diff --git a/lib/handlebars/compiler/ast.js b/lib/handlebars/compiler/ast.js
index 850c60554..c1789608d 100644
--- a/lib/handlebars/compiler/ast.js
+++ b/lib/handlebars/compiler/ast.js
@@ -67,21 +67,24 @@ Handlebars.AST.HashNode = function(pairs) {
 
 Handlebars.AST.IdNode = function(parts) {
   this.type = "ID";
-  this.original = parts.join(".");
 
-  var dig = [], depth = 0;
+  var original = "",
+      dig = [],
+      depth = 0;
 
   for(var i=0,l=parts.length; i<l; i++) {
-    var part = parts[i];
+    var part = parts[i].part;
+    original += (parts[i].separator || '') + part;
 
     if (part === ".." || part === "." || part === "this") {
-      if (dig.length > 0) { throw new Handlebars.Exception("Invalid path: " + this.original); }
+      if (dig.length > 0) { throw new Handlebars.Exception("Invalid path: " + original); }
       else if (part === "..") { depth++; }
       else { this.isScoped = true; }
     }
     else { dig.push(part); }
   }
 
+  this.original = original;
   this.parts    = dig;
   this.string   = dig.join('.');
   this.depth    = depth;
@@ -105,13 +108,15 @@ Handlebars.AST.DataNode = function(id) {
 
 Handlebars.AST.StringNode = function(string) {
   this.type = "STRING";
-  this.string = string;
-  this.stringModeValue = string;
+  this.original =
+    this.string =
+    this.stringModeValue = string;
 };
 
 Handlebars.AST.IntegerNode = function(integer) {
   this.type = "INTEGER";
-  this.integer = integer;
+  this.original =
+    this.integer = integer;
   this.stringModeValue = Number(integer);
 };
 
diff --git a/src/handlebars.yy b/src/handlebars.yy
index 7e6321f8b..4e3a82634 100644
--- a/src/handlebars.yy
+++ b/src/handlebars.yy
@@ -107,7 +107,7 @@ path
   ;
 
 pathSegments
-  : pathSegments SEP ID { $1.push($3); $$ = $1; }
-  | ID { $$ = [$1]; }
+  : pathSegments SEP ID { $1.push({part: $3, separator: $2}); $$ = $1; }
+  | ID { $$ = [{part: $1}]; }
   ;
 

From 3ddbc5237f5208f40a7d438895fe2249bdf343b9 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Tue, 28 May 2013 17:03:07 -0400
Subject: [PATCH 21/34] Protect from object prototype modifications

Fixes #534
---
 dist/handlebars.js                  | 4 +++-
 lib/handlebars/compiler/compiler.js | 4 +++-
 release-notes.md                    | 1 +
 spec/qunit_spec.js                  | 8 ++++++++
 4 files changed, 15 insertions(+), 2 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 56b6fea8a..023c9ef11 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -1406,7 +1406,9 @@ JavaScriptCompiler.prototype = {
     // Generate minimizer alias mappings
     if (!this.isChild) {
       for (var alias in this.context.aliases) {
-        this.source[1] = this.source[1] + ', ' + alias + '=' + this.context.aliases[alias];
+        if (this.context.aliases.hasOwnProperty(alias)) {
+          this.source[1] = this.source[1] + ', ' + alias + '=' + this.context.aliases[alias];
+        }
       }
     }
 
diff --git a/lib/handlebars/compiler/compiler.js b/lib/handlebars/compiler/compiler.js
index d40e5f88f..7777456ce 100644
--- a/lib/handlebars/compiler/compiler.js
+++ b/lib/handlebars/compiler/compiler.js
@@ -538,7 +538,9 @@ JavaScriptCompiler.prototype = {
     // Generate minimizer alias mappings
     if (!this.isChild) {
       for (var alias in this.context.aliases) {
-        this.source[1] = this.source[1] + ', ' + alias + '=' + this.context.aliases[alias];
+        if (this.context.aliases.hasOwnProperty(alias)) {
+          this.source[1] = this.source[1] + ', ' + alias + '=' + this.context.aliases[alias];
+        }
       }
     }
 
diff --git a/release-notes.md b/release-notes.md
index b233bd621..ae0a325ce 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -6,6 +6,7 @@
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
 - [#433](https://github.com/wycats/handlebars.js/issues/433) - Add support for unicode ids
 - [#469](https://github.com/wycats/handlebars.js/issues/469) - Add support for `?` in ids
+- [#534](https://github.com/wycats/handlebars.js/issues/534) - Protect from object prototype modifications
 - Add support for complex ids in @data references
 - Docs updates
 
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 23dcdb9ae..5f14db0f6 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -1530,6 +1530,14 @@ test('GH-375: Unicode line terminators', function() {
   shouldCompileTo('\u2028', {}, '\u2028');
 });
 
+test('GH-534: Object prototype aliases', function() {
+  Object.prototype[0xD834] = true;
+
+  shouldCompileTo('{{foo}}', { foo: 'bar' }, 'bar');
+
+  delete Object.prototype[0xD834];
+});
+
 suite('Utils');
 
 test('escapeExpression', function() {

From da2aabe7bdc75e29178d920332fad9de183de8a0 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 10:46:28 -0400
Subject: [PATCH 22/34] Allow ID, STRING, or INTEGER for partial name

Fixes #519
---
 dist/handlebars.js             | 40 +++++++++++++++++-----------------
 lib/handlebars/compiler/ast.js |  2 +-
 release-notes.md               |  3 ++-
 spec/parser_spec.rb            |  2 +-
 spec/qunit_spec.js             | 20 +++++++++++++++++
 spec/tokenizer_spec.rb         | 21 +++++++++++-------
 src/handlebars.l               |  6 ++---
 src/handlebars.yy              |  4 +++-
 8 files changed, 62 insertions(+), 36 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 023c9ef11..da556ddcc 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -181,9 +181,9 @@ Handlebars.registerHelper('log', function(context, options) {
 var handlebars = (function(){
 var parser = {trace: function trace() { },
 yy: {},
-symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"OPEN_PARTIAL":24,"partialName":25,"params":26,"hash":27,"dataName":28,"param":29,"STRING":30,"INTEGER":31,"BOOLEAN":32,"hashSegments":33,"hashSegment":34,"ID":35,"EQUALS":36,"PARTIAL_NAME":37,"DATA":38,"pathSegments":39,"SEP":40,"$accept":0,"$end":1},
-terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"OPEN_PARTIAL",30:"STRING",31:"INTEGER",32:"BOOLEAN",35:"ID",36:"EQUALS",37:"PARTIAL_NAME",38:"DATA",40:"SEP"},
-productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[26,2],[26,1],[29,1],[29,1],[29,1],[29,1],[29,1],[27,1],[33,2],[33,1],[34,3],[34,3],[34,3],[34,3],[34,3],[25,1],[28,2],[21,1],[39,3],[39,1]],
+symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"OPEN_PARTIAL":24,"partialName":25,"params":26,"hash":27,"dataName":28,"param":29,"STRING":30,"INTEGER":31,"BOOLEAN":32,"hashSegments":33,"hashSegment":34,"ID":35,"EQUALS":36,"DATA":37,"pathSegments":38,"SEP":39,"$accept":0,"$end":1},
+terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"OPEN_PARTIAL",30:"STRING",31:"INTEGER",32:"BOOLEAN",35:"ID",36:"EQUALS",37:"DATA",39:"SEP"},
+productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[26,2],[26,1],[29,1],[29,1],[29,1],[29,1],[29,1],[27,1],[33,2],[33,1],[34,3],[34,3],[34,3],[34,3],[34,3],[25,1],[25,1],[25,1],[28,2],[21,1],[38,3],[38,1]],
 performAction: function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$) {
 
 var $0 = $$.length - 1;
@@ -276,18 +276,22 @@ case 43: this.$ = [$$[$0-2], $$[$0]];
 break;
 case 44: this.$ = new yy.PartialNameNode($$[$0]); 
 break;
-case 45: this.$ = new yy.DataNode($$[$0]); 
+case 45: this.$ = new yy.PartialNameNode(new yy.StringNode($$[$0])); 
 break;
-case 46: this.$ = new yy.IdNode($$[$0]); 
+case 46: this.$ = new yy.PartialNameNode(new yy.IntegerNode($$[$0])); 
 break;
-case 47: $$[$0-2].push({part: $$[$0], separator: $$[$0-1]}); this.$ = $$[$0-2]; 
+case 47: this.$ = new yy.DataNode($$[$0]); 
 break;
-case 48: this.$ = [{part: $$[$0]}]; 
+case 48: this.$ = new yy.IdNode($$[$0]); 
+break;
+case 49: $$[$0-2].push({part: $$[$0], separator: $$[$0-1]}); this.$ = $$[$0-2]; 
+break;
+case 50: this.$ = [{part: $$[$0]}]; 
 break;
 }
 },
-table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],24:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],24:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],24:[1,16]},{17:23,18:[1,22],21:24,28:25,35:[1,28],38:[1,27],39:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],24:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],24:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],24:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],24:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],24:[2,15]},{17:31,21:24,28:25,35:[1,28],38:[1,27],39:26},{17:32,21:24,28:25,35:[1,28],38:[1,27],39:26},{17:33,21:24,28:25,35:[1,28],38:[1,27],39:26},{25:34,37:[1,35]},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],24:[1,16]},{17:23,21:24,28:25,35:[1,28],38:[1,27],39:26},{5:[2,4],7:36,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],24:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],24:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],24:[2,23]},{18:[1,37]},{18:[2,27],21:42,26:38,27:39,28:46,29:40,30:[1,43],31:[1,44],32:[1,45],33:41,34:47,35:[1,48],38:[1,27],39:26},{18:[2,28]},{18:[2,46],30:[2,46],31:[2,46],32:[2,46],35:[2,46],38:[2,46],40:[1,49]},{21:50,35:[1,28],39:26},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],38:[2,48],40:[2,48]},{10:51,20:[1,52]},{10:53,20:[1,52]},{18:[1,54]},{18:[1,55]},{18:[1,56]},{18:[1,57],21:58,35:[1,28],39:26},{18:[2,44],35:[2,44]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],24:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],24:[2,17]},{18:[2,25],21:42,27:59,28:46,29:60,30:[1,43],31:[1,44],32:[1,45],33:41,34:47,35:[1,48],38:[1,27],39:26},{18:[2,26]},{18:[2,30],30:[2,30],31:[2,30],32:[2,30],35:[2,30],38:[2,30]},{18:[2,36],34:61,35:[1,62]},{18:[2,31],30:[2,31],31:[2,31],32:[2,31],35:[2,31],38:[2,31]},{18:[2,32],30:[2,32],31:[2,32],32:[2,32],35:[2,32],38:[2,32]},{18:[2,33],30:[2,33],31:[2,33],32:[2,33],35:[2,33],38:[2,33]},{18:[2,34],30:[2,34],31:[2,34],32:[2,34],35:[2,34],38:[2,34]},{18:[2,35],30:[2,35],31:[2,35],32:[2,35],35:[2,35],38:[2,35]},{18:[2,38],35:[2,38]},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],36:[1,63],38:[2,48],40:[2,48]},{35:[1,64]},{18:[2,45],30:[2,45],31:[2,45],32:[2,45],35:[2,45],38:[2,45]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],24:[2,10]},{21:65,35:[1,28],39:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],24:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],24:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],24:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],24:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],24:[2,21]},{18:[1,66]},{18:[2,24]},{18:[2,29],30:[2,29],31:[2,29],32:[2,29],35:[2,29],38:[2,29]},{18:[2,37],35:[2,37]},{36:[1,63]},{21:67,28:71,30:[1,68],31:[1,69],32:[1,70],35:[1,28],38:[1,27],39:26},{18:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],38:[2,47],40:[2,47]},{18:[1,72]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],24:[2,22]},{18:[2,39],35:[2,39]},{18:[2,40],35:[2,40]},{18:[2,41],35:[2,41]},{18:[2,42],35:[2,42]},{18:[2,43],35:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],24:[2,18]}],
-defaultActions: {17:[2,1],25:[2,28],39:[2,26],59:[2,24]},
+table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],24:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],24:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],24:[1,16]},{17:23,18:[1,22],21:24,28:25,35:[1,28],37:[1,27],38:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],24:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],24:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],24:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],24:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],24:[2,15]},{17:31,21:24,28:25,35:[1,28],37:[1,27],38:26},{17:32,21:24,28:25,35:[1,28],37:[1,27],38:26},{17:33,21:24,28:25,35:[1,28],37:[1,27],38:26},{21:35,25:34,30:[1,36],31:[1,37],35:[1,28],38:26},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],24:[1,16]},{17:23,21:24,28:25,35:[1,28],37:[1,27],38:26},{5:[2,4],7:38,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],24:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],24:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],24:[2,23]},{18:[1,39]},{18:[2,27],21:44,26:40,27:41,28:48,29:42,30:[1,45],31:[1,46],32:[1,47],33:43,34:49,35:[1,50],37:[1,27],38:26},{18:[2,28]},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],37:[2,48],39:[1,51]},{21:52,35:[1,28],38:26},{18:[2,50],30:[2,50],31:[2,50],32:[2,50],35:[2,50],37:[2,50],39:[2,50]},{10:53,20:[1,54]},{10:55,20:[1,54]},{18:[1,56]},{18:[1,57]},{18:[1,58]},{18:[1,59],21:60,35:[1,28],38:26},{18:[2,44],35:[2,44]},{18:[2,45],35:[2,45]},{18:[2,46],35:[2,46]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],24:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],24:[2,17]},{18:[2,25],21:44,27:61,28:48,29:62,30:[1,45],31:[1,46],32:[1,47],33:43,34:49,35:[1,50],37:[1,27],38:26},{18:[2,26]},{18:[2,30],30:[2,30],31:[2,30],32:[2,30],35:[2,30],37:[2,30]},{18:[2,36],34:63,35:[1,64]},{18:[2,31],30:[2,31],31:[2,31],32:[2,31],35:[2,31],37:[2,31]},{18:[2,32],30:[2,32],31:[2,32],32:[2,32],35:[2,32],37:[2,32]},{18:[2,33],30:[2,33],31:[2,33],32:[2,33],35:[2,33],37:[2,33]},{18:[2,34],30:[2,34],31:[2,34],32:[2,34],35:[2,34],37:[2,34]},{18:[2,35],30:[2,35],31:[2,35],32:[2,35],35:[2,35],37:[2,35]},{18:[2,38],35:[2,38]},{18:[2,50],30:[2,50],31:[2,50],32:[2,50],35:[2,50],36:[1,65],37:[2,50],39:[2,50]},{35:[1,66]},{18:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],37:[2,47]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],24:[2,10]},{21:67,35:[1,28],38:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],24:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],24:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],24:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],24:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],24:[2,21]},{18:[1,68]},{18:[2,24]},{18:[2,29],30:[2,29],31:[2,29],32:[2,29],35:[2,29],37:[2,29]},{18:[2,37],35:[2,37]},{36:[1,65]},{21:69,28:73,30:[1,70],31:[1,71],32:[1,72],35:[1,28],37:[1,27],38:26},{18:[2,49],30:[2,49],31:[2,49],32:[2,49],35:[2,49],37:[2,49],39:[2,49]},{18:[1,74]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],24:[2,22]},{18:[2,39],35:[2,39]},{18:[2,40],35:[2,40]},{18:[2,41],35:[2,41]},{18:[2,42],35:[2,42]},{18:[2,43],35:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],24:[2,18]}],
+defaultActions: {17:[2,1],25:[2,28],41:[2,26],61:[2,24]},
 parseError: function parseError(str, hash) {
     throw new Error(str);
 },
@@ -586,7 +590,7 @@ case 3:
 break;
 case 4: yy_.yytext = yy_.yytext.substr(0, yy_.yyleng-4); this.popState(); return 15; 
 break;
-case 5: this.begin("par"); return 24; 
+case 5: return 24; 
 break;
 case 6: return 16; 
 break;
@@ -612,7 +616,7 @@ case 16: return 35;
 break;
 case 17: return 35; 
 break;
-case 18: return 40; 
+case 18: return 39; 
 break;
 case 19: /*ignore whitespace*/ 
 break;
@@ -624,7 +628,7 @@ case 22: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\"/g,'"'); ret
 break;
 case 23: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\'/g,"'"); return 30; 
 break;
-case 24: return 38; 
+case 24: return 37; 
 break;
 case 25: return 32; 
 break;
@@ -638,16 +642,12 @@ case 29: yy_.yytext = yy_.yytext.substr(1, yy_.yyleng-2); return 35;
 break;
 case 30: return 'INVALID'; 
 break;
-case 31: /*ignore whitespace*/ 
-break;
-case 32: this.popState(); return 37; 
-break;
-case 33: return 5; 
+case 31: return 5; 
 break;
 }
 };
-lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:\s+)/,/^(?:[a-zA-Z0-9_$\-\/\.]+)/,/^(?:$)/];
-lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,33],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"par":{"rules":[31,32],"inclusive":false},"INITIAL":{"rules":[0,1,2,33],"inclusive":true}};
+lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:$)/];
+lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"INITIAL":{"rules":[0,1,2,31],"inclusive":true}};
 return lexer;})()
 parser.lexer = lexer;
 function Parser () { this.yy = {}; }Parser.prototype = parser;parser.Parser = Parser;
@@ -764,7 +764,7 @@ Handlebars.AST.IdNode = function(parts) {
 
 Handlebars.AST.PartialNameNode = function(name) {
   this.type = "PARTIAL_NAME";
-  this.name = name;
+  this.name = name.original;
 };
 
 Handlebars.AST.DataNode = function(id) {
diff --git a/lib/handlebars/compiler/ast.js b/lib/handlebars/compiler/ast.js
index c1789608d..c99728ccd 100644
--- a/lib/handlebars/compiler/ast.js
+++ b/lib/handlebars/compiler/ast.js
@@ -98,7 +98,7 @@ Handlebars.AST.IdNode = function(parts) {
 
 Handlebars.AST.PartialNameNode = function(name) {
   this.type = "PARTIAL_NAME";
-  this.name = name;
+  this.name = name.original;
 };
 
 Handlebars.AST.DataNode = function(id) {
diff --git a/release-notes.md b/release-notes.md
index ae0a325ce..5a31418a9 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -3,10 +3,11 @@
 ## Development
 - [#515](https://github.com/wycats/handlebars.js/issues/515) - Add node require extensions support ([@jjclark1982](https://github.com/jjclark1982))
 - [#517](https://github.com/wycats/handlebars.js/issues/517) - Fix amd precompiler output with directories ([@blessenm](https://github.com/blessenm))
-- [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
 - [#433](https://github.com/wycats/handlebars.js/issues/433) - Add support for unicode ids
 - [#469](https://github.com/wycats/handlebars.js/issues/469) - Add support for `?` in ids
 - [#534](https://github.com/wycats/handlebars.js/issues/534) - Protect from object prototype modifications
+- [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
+- [#519](https://github.com/wycats/handlebars.js/issues/519) - Allow ID or strings in partial names
 - Add support for complex ids in @data references
 - Docs updates
 
diff --git a/spec/parser_spec.rb b/spec/parser_spec.rb
index e478aa5b8..3ee011778 100644
--- a/spec/parser_spec.rb
+++ b/spec/parser_spec.rb
@@ -234,7 +234,7 @@ def path(*parts)
   end
 
   it "parses a partial with a complex name" do
-    ast_for("{{> shared/partial}}").should == root { partial partial_name("shared/partial") }
+    ast_for("{{> shared/partial?.bar}}").should == root { partial partial_name("shared/partial?.bar") }
   end
 
   it "parses a comment" do
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 5f14db0f6..933c8ed65 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -602,6 +602,26 @@ test("Partials with integer path", function() {
   shouldCompileToWithPartials(string, [hash, {}, {404:dude}], true, "Dudes: Jeepers", "Partials can use literal paths");
 });
 
+test("Partials with complex path", function() {
+  var string = "Dudes: {{> 404/asdf?.bar}}";
+  var dude = "{{name}}";
+  var hash = {name:"Jeepers", another_dude:"Creepers"};
+  shouldCompileToWithPartials(string, [hash, {}, {'404/asdf?.bar':dude}], true, "Dudes: Jeepers", "Partials can use literal paths");
+});
+
+test("Partials with escaped", function() {
+  var string = "Dudes: {{> [+404/asdf?.bar]}}";
+  var dude = "{{name}}";
+  var hash = {name:"Jeepers", another_dude:"Creepers"};
+  shouldCompileToWithPartials(string, [hash, {}, {'+404/asdf?.bar':dude}], true, "Dudes: Jeepers", "Partials can use literal paths");
+});
+
+test("Partials with string", function() {
+  var string = "Dudes: {{> \"+404/asdf?.bar\"}}";
+  var dude = "{{name}}";
+  var hash = {name:"Jeepers", another_dude:"Creepers"};
+  shouldCompileToWithPartials(string, [hash, {}, {'+404/asdf?.bar':dude}], true, "Dudes: Jeepers", "Partials can use literal paths");
+});
 
 suite("String literal parameters");
 
diff --git a/spec/tokenizer_spec.rb b/spec/tokenizer_spec.rb
index c090b527b..fc7a3acaa 100644
--- a/spec/tokenizer_spec.rb
+++ b/spec/tokenizer_spec.rb
@@ -143,24 +143,29 @@ def tokenize(string)
     result[4].should be_token("CONTENT", " baz")
   end
 
-  it "tokenizes a partial as 'OPEN_PARTIAL PARTIAL_NAME CLOSE'" do
+  it "tokenizes a partial as 'OPEN_PARTIAL ID CLOSE'" do
     result = tokenize("{{> foo}}")
-    result.should match_tokens(%w(OPEN_PARTIAL PARTIAL_NAME CLOSE))
+    result.should match_tokens(%w(OPEN_PARTIAL ID CLOSE))
   end
 
-  it "tokenizes a partial with context as 'OPEN_PARTIAL PARTIAL_NAME ID CLOSE'" do
+  it "tokenizes a partial with context as 'OPEN_PARTIAL ID ID CLOSE'" do
     result = tokenize("{{> foo bar }}")
-    result.should match_tokens(%w(OPEN_PARTIAL PARTIAL_NAME ID CLOSE))
+    result.should match_tokens(%w(OPEN_PARTIAL ID ID CLOSE))
   end
 
-  it "tokenizes a partial without spaces as 'OPEN_PARTIAL PARTIAL_NAME CLOSE'" do
+  it "tokenizes a partial without spaces as 'OPEN_PARTIAL ID CLOSE'" do
     result = tokenize("{{>foo}}")
-    result.should match_tokens(%w(OPEN_PARTIAL PARTIAL_NAME CLOSE))
+    result.should match_tokens(%w(OPEN_PARTIAL ID CLOSE))
   end
 
-  it "tokenizes a partial space at the end as 'OPEN_PARTIAL PARTIAL_NAME CLOSE'" do
+  it "tokenizes a partial space at the end as 'OPEN_PARTIAL ID CLOSE'" do
     result = tokenize("{{>foo  }}")
-    result.should match_tokens(%w(OPEN_PARTIAL PARTIAL_NAME CLOSE))
+    result.should match_tokens(%w(OPEN_PARTIAL ID CLOSE))
+  end
+
+  it "tokenizes a partial space at the end as 'OPEN_PARTIAL ID CLOSE'" do
+    result = tokenize("{{>foo/bar.baz  }}")
+    result.should match_tokens(%w(OPEN_PARTIAL ID SEP ID SEP ID CLOSE))
   end
 
   it "tokenizes a comment as 'COMMENT'" do
diff --git a/src/handlebars.l b/src/handlebars.l
index 7fcf86e30..2afd715bb 100644
--- a/src/handlebars.l
+++ b/src/handlebars.l
@@ -1,5 +1,5 @@
 
-%x mu emu com par
+%x mu emu com
 
 %%
 
@@ -20,7 +20,7 @@
 
 <com>[\s\S]*?"--}}"              { yytext = yytext.substr(0, yyleng-4); this.popState(); return 'COMMENT'; }
 
-<mu>"{{>"                        { this.begin("par"); return 'OPEN_PARTIAL'; }
+<mu>"{{>"                        { return 'OPEN_PARTIAL'; }
 <mu>"{{#"                        { return 'OPEN_BLOCK'; }
 <mu>"{{/"                        { return 'OPEN_ENDBLOCK'; }
 <mu>"{{^"                        { return 'OPEN_INVERSE'; }
@@ -58,8 +58,6 @@ Control characters ranges:
 
 <mu>'['[^\]]*']'                 { yytext = yytext.substr(1, yyleng-2); return 'ID'; }
 <mu>.                            { return 'INVALID'; }
-<par>\s+                         { /*ignore whitespace*/ }
-<par>[a-zA-Z0-9_$\-\/\.]+        { this.popState(); return 'PARTIAL_NAME'; }
 
 <INITIAL,mu><<EOF>>              { return 'EOF'; }
 
diff --git a/src/handlebars.yy b/src/handlebars.yy
index 4e3a82634..e521c2e5e 100644
--- a/src/handlebars.yy
+++ b/src/handlebars.yy
@@ -95,7 +95,9 @@ hashSegment
   ;
 
 partialName
-  : PARTIAL_NAME { $$ = new yy.PartialNameNode($1); }
+  : path { $$ = new yy.PartialNameNode($1); }
+  | STRING { $$ = new yy.PartialNameNode(new yy.StringNode($1)); }
+  | INTEGER { $$ = new yy.PartialNameNode(new yy.IntegerNode($1)); }
   ;
 
 dataName

From 293672432b3912a1d1f6b659a7e08334532f56d5 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 10:57:48 -0400
Subject: [PATCH 23/34] improve error message on missing helper

Via @stefanpenner
Fixes #523
---
 dist/handlebars.js         | 2 +-
 dist/handlebars.runtime.js | 2 +-
 lib/handlebars/base.js     | 2 +-
 spec/qunit_spec.js         | 2 +-
 4 files changed, 4 insertions(+), 4 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index da556ddcc..5139dc8a3 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -67,7 +67,7 @@ Handlebars.registerHelper('helperMissing', function(arg) {
   if(arguments.length === 2) {
     return undefined;
   } else {
-    throw new Error("Could not find property '" + arg + "'");
+    throw new Error("Missing helper: '" + arg + "'");
   }
 });
 
diff --git a/dist/handlebars.runtime.js b/dist/handlebars.runtime.js
index 8744aa77b..ac2b020ea 100644
--- a/dist/handlebars.runtime.js
+++ b/dist/handlebars.runtime.js
@@ -67,7 +67,7 @@ Handlebars.registerHelper('helperMissing', function(arg) {
   if(arguments.length === 2) {
     return undefined;
   } else {
-    throw new Error("Could not find property '" + arg + "'");
+    throw new Error("Missing helper: '" + arg + "'");
   }
 });
 
diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index 0a9b18cd2..d7284b0d1 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -44,7 +44,7 @@ Handlebars.registerHelper('helperMissing', function(arg) {
   if(arguments.length === 2) {
     return undefined;
   } else {
-    throw new Error("Could not find property '" + arg + "'");
+    throw new Error("Missing helper: '" + arg + "'");
   }
 });
 
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 933c8ed65..6a093f373 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -706,7 +706,7 @@ test("if a context is not found, helperMissing is used", function() {
   shouldThrow(function() {
       var template = CompilerContext.compile("{{hello}} {{link_to world}}");
       template({});
-    }, [Error, "Could not find property 'link_to'"], "Should throw exception");
+    }, [Error, "Missing helper: 'link_to'"], "Should throw exception");
 });
 
 test("if a context is not found, custom helperMissing is used", function() {

From c540d7186adb28db3cbd2923cbf82d60f44f777e Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 12:23:55 -0400
Subject: [PATCH 24/34] Allow function arguments to with and each

Via @mcdan
Fixes #239
---
 dist/handlebars.js         |  6 ++++++
 dist/handlebars.runtime.js |  6 ++++++
 lib/handlebars/base.js     |  6 ++++++
 spec/qunit_spec.js         | 13 +++++++++++++
 4 files changed, 31 insertions(+)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 5139dc8a3..95da9e4db 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -124,6 +124,9 @@ Handlebars.registerHelper('each', function(context, options) {
   var fn = options.fn, inverse = options.inverse;
   var i = 0, ret = "", data;
 
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (options.data) {
     data = Handlebars.createFrame(options.data);
   }
@@ -168,6 +171,9 @@ Handlebars.registerHelper('unless', function(conditional, options) {
 });
 
 Handlebars.registerHelper('with', function(context, options) {
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (!Handlebars.Utils.isEmpty(context)) return options.fn(context);
 });
 
diff --git a/dist/handlebars.runtime.js b/dist/handlebars.runtime.js
index ac2b020ea..d11c79d54 100644
--- a/dist/handlebars.runtime.js
+++ b/dist/handlebars.runtime.js
@@ -124,6 +124,9 @@ Handlebars.registerHelper('each', function(context, options) {
   var fn = options.fn, inverse = options.inverse;
   var i = 0, ret = "", data;
 
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (options.data) {
     data = Handlebars.createFrame(options.data);
   }
@@ -168,6 +171,9 @@ Handlebars.registerHelper('unless', function(conditional, options) {
 });
 
 Handlebars.registerHelper('with', function(context, options) {
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (!Handlebars.Utils.isEmpty(context)) return options.fn(context);
 });
 
diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index d7284b0d1..ad2d9ec2c 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -101,6 +101,9 @@ Handlebars.registerHelper('each', function(context, options) {
   var fn = options.fn, inverse = options.inverse;
   var i = 0, ret = "", data;
 
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (options.data) {
     data = Handlebars.createFrame(options.data);
   }
@@ -145,6 +148,9 @@ Handlebars.registerHelper('unless', function(conditional, options) {
 });
 
 Handlebars.registerHelper('with', function(context, options) {
+  var type = toString.call(context);
+  if(type === functionType) { context = context.call(this); }
+
   if (!Handlebars.Utils.isEmpty(context)) return options.fn(context);
 });
 
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 6a093f373..31c9b96a4 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -797,6 +797,10 @@ test("with", function() {
   var string = "{{#with person}}{{first}} {{last}}{{/with}}";
   shouldCompileTo(string, {person: {first: "Alan", last: "Johnson"}}, "Alan Johnson");
 });
+test("with with function argument", function() {
+  var string = "{{#with person}}{{first}} {{last}}{{/with}}";
+  shouldCompileTo(string, {person: function() { return {first: "Alan", last: "Johnson"};}}, "Alan Johnson");
+});
 
 test("if", function() {
   var string   = "{{#if goodbye}}GOODBYE {{/if}}cruel {{world}}!";
@@ -861,6 +865,15 @@ test("each with @index", function() {
   equal(result, "0. goodbye! 1. Goodbye! 2. GOODBYE! cruel world!", "The @index variable is used");
 });
 
+test("each with function argument", function() {
+  var string = "{{#each goodbyes}}{{text}}! {{/each}}cruel {{world}}!";
+  var hash   = {goodbyes: function () { return [{text: "goodbye"}, {text: "Goodbye"}, {text: "GOODBYE"}];}, world: "world"};
+  shouldCompileTo(string, hash, "goodbye! Goodbye! GOODBYE! cruel world!",
+            "each with array function argument iterates over the contents when not empty");
+  shouldCompileTo(string, {goodbyes: [], world: "world"}, "cruel world!",
+            "each with array function argument ignores the contents when empty");
+});
+
 test("data passed to helpers", function() {
   var string = "{{#each letters}}{{this}}{{detectDataInsideEach}}{{/each}}";
   var hash = {letters: ['a', 'b', 'c']};

From 66f6f94867e622e9df10a875a9c8a2fda2a82565 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 15:17:10 -0400
Subject: [PATCH 25/34] Update to latest uglify lib

---
 bin/handlebars | 5 +----
 package.json   | 2 +-
 2 files changed, 2 insertions(+), 5 deletions(-)

diff --git a/bin/handlebars b/bin/handlebars
index 5b1ba89c9..4835770f9 100755
--- a/bin/handlebars
+++ b/bin/handlebars
@@ -203,10 +203,7 @@ if (!argv.simple) {
 output = output.join('');
 
 if (argv.min) {
-  var ast = uglify.parser.parse(output);
-  ast = uglify.uglify.ast_mangle(ast);
-  ast = uglify.uglify.ast_squeeze(ast);
-  output = uglify.uglify.gen_code(ast);
+  output = uglify.minify(output, {fromString: true}).code;
 }
 
 if (argv.output) {
diff --git a/package.json b/package.json
index ab3f6831e..af8db6855 100644
--- a/package.json
+++ b/package.json
@@ -15,7 +15,7 @@
   },
   "dependencies": {
     "optimist": "~0.3",
-    "uglify-js": "~1.2"
+    "uglify-js": "~2.3"
   },
   "devDependencies": {
     "benchmark": "~1.0",

From 61f64e9f04ec6039759519371aff76cd504ed5da Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 15:21:19 -0400
Subject: [PATCH 26/34] Add minify task to release target

---
 Rakefile | 7 ++++++-
 1 file changed, 6 insertions(+), 1 deletion(-)

diff --git a/Rakefile b/Rakefile
index a6e3e102b..794686f1e 100644
--- a/Rakefile
+++ b/Rakefile
@@ -106,8 +106,13 @@ task :version => [] do |task|
   end
 end
 
+task :minify => [] do |task|
+  system "./node_modules/.bin/uglifyjs --comments -o dist/handlebars.min.js dist/handlebars.js"
+  system "./node_modules/.bin/uglifyjs --comments -o dist/handlebars.runtime.min.js dist/handlebars.runtime.js"
+end
+
 desc "build the build and runtime version of handlebars"
-task :release => [:version, :build, :runtime]
+task :release => [:version, :build, :runtime, :minify]
 
 directory "vendor"
 

From 58a0b4f17d5338793c92cf4d104e9c44cc485c5b Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 15:30:43 -0400
Subject: [PATCH 27/34] Require matching braces in escaped expressions

Fixes #437
---
 dist/handlebars.js     | 45 ++++++++++++++++++++++--------------------
 release-notes.md       |  5 +++++
 spec/qunit_spec.js     |  9 +++++++++
 spec/tokenizer_spec.rb |  5 +++--
 src/handlebars.l       |  4 ++--
 src/handlebars.yy      |  7 +++++--
 6 files changed, 48 insertions(+), 27 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 95da9e4db..a5f0bf7e6 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -187,9 +187,9 @@ Handlebars.registerHelper('log', function(context, options) {
 var handlebars = (function(){
 var parser = {trace: function trace() { },
 yy: {},
-symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"OPEN_PARTIAL":24,"partialName":25,"params":26,"hash":27,"dataName":28,"param":29,"STRING":30,"INTEGER":31,"BOOLEAN":32,"hashSegments":33,"hashSegment":34,"ID":35,"EQUALS":36,"DATA":37,"pathSegments":38,"SEP":39,"$accept":0,"$end":1},
-terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"OPEN_PARTIAL",30:"STRING",31:"INTEGER",32:"BOOLEAN",35:"ID",36:"EQUALS",37:"DATA",39:"SEP"},
-productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[26,2],[26,1],[29,1],[29,1],[29,1],[29,1],[29,1],[27,1],[33,2],[33,1],[34,3],[34,3],[34,3],[34,3],[34,3],[25,1],[25,1],[25,1],[28,2],[21,1],[38,3],[38,1]],
+symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"CLOSE_UNESCAPED":24,"OPEN_PARTIAL":25,"partialName":26,"params":27,"hash":28,"dataName":29,"param":30,"STRING":31,"INTEGER":32,"BOOLEAN":33,"hashSegments":34,"hashSegment":35,"ID":36,"EQUALS":37,"DATA":38,"pathSegments":39,"SEP":40,"$accept":0,"$end":1},
+terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"CLOSE_UNESCAPED",25:"OPEN_PARTIAL",31:"STRING",32:"INTEGER",33:"BOOLEAN",36:"ID",37:"EQUALS",38:"DATA",40:"SEP"},
+productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[27,2],[27,1],[30,1],[30,1],[30,1],[30,1],[30,1],[28,1],[34,2],[34,1],[35,3],[35,3],[35,3],[35,3],[35,3],[26,1],[26,1],[26,1],[29,2],[21,1],[39,3],[39,1]],
 performAction: function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$) {
 
 var $0 = $$.length - 1;
@@ -230,7 +230,10 @@ case 17: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1]);
 break;
 case 18: this.$ = $$[$0-1]; 
 break;
-case 19: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1]); 
+case 19:
+    // Parsing out the '&' escape token at this level saves ~500 bytes after min due to the removal of one parser node.
+    this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1], $$[$0-2][2] === '&');
+  
 break;
 case 20: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1], true); 
 break;
@@ -296,8 +299,8 @@ case 50: this.$ = [{part: $$[$0]}];
 break;
 }
 },
-table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],24:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],24:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],24:[1,16]},{17:23,18:[1,22],21:24,28:25,35:[1,28],37:[1,27],38:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],24:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],24:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],24:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],24:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],24:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],24:[2,15]},{17:31,21:24,28:25,35:[1,28],37:[1,27],38:26},{17:32,21:24,28:25,35:[1,28],37:[1,27],38:26},{17:33,21:24,28:25,35:[1,28],37:[1,27],38:26},{21:35,25:34,30:[1,36],31:[1,37],35:[1,28],38:26},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],24:[1,16]},{17:23,21:24,28:25,35:[1,28],37:[1,27],38:26},{5:[2,4],7:38,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],24:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],24:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],24:[2,23]},{18:[1,39]},{18:[2,27],21:44,26:40,27:41,28:48,29:42,30:[1,45],31:[1,46],32:[1,47],33:43,34:49,35:[1,50],37:[1,27],38:26},{18:[2,28]},{18:[2,48],30:[2,48],31:[2,48],32:[2,48],35:[2,48],37:[2,48],39:[1,51]},{21:52,35:[1,28],38:26},{18:[2,50],30:[2,50],31:[2,50],32:[2,50],35:[2,50],37:[2,50],39:[2,50]},{10:53,20:[1,54]},{10:55,20:[1,54]},{18:[1,56]},{18:[1,57]},{18:[1,58]},{18:[1,59],21:60,35:[1,28],38:26},{18:[2,44],35:[2,44]},{18:[2,45],35:[2,45]},{18:[2,46],35:[2,46]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],24:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],24:[2,17]},{18:[2,25],21:44,27:61,28:48,29:62,30:[1,45],31:[1,46],32:[1,47],33:43,34:49,35:[1,50],37:[1,27],38:26},{18:[2,26]},{18:[2,30],30:[2,30],31:[2,30],32:[2,30],35:[2,30],37:[2,30]},{18:[2,36],34:63,35:[1,64]},{18:[2,31],30:[2,31],31:[2,31],32:[2,31],35:[2,31],37:[2,31]},{18:[2,32],30:[2,32],31:[2,32],32:[2,32],35:[2,32],37:[2,32]},{18:[2,33],30:[2,33],31:[2,33],32:[2,33],35:[2,33],37:[2,33]},{18:[2,34],30:[2,34],31:[2,34],32:[2,34],35:[2,34],37:[2,34]},{18:[2,35],30:[2,35],31:[2,35],32:[2,35],35:[2,35],37:[2,35]},{18:[2,38],35:[2,38]},{18:[2,50],30:[2,50],31:[2,50],32:[2,50],35:[2,50],36:[1,65],37:[2,50],39:[2,50]},{35:[1,66]},{18:[2,47],30:[2,47],31:[2,47],32:[2,47],35:[2,47],37:[2,47]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],24:[2,10]},{21:67,35:[1,28],38:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],24:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],24:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],24:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],24:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],24:[2,21]},{18:[1,68]},{18:[2,24]},{18:[2,29],30:[2,29],31:[2,29],32:[2,29],35:[2,29],37:[2,29]},{18:[2,37],35:[2,37]},{36:[1,65]},{21:69,28:73,30:[1,70],31:[1,71],32:[1,72],35:[1,28],37:[1,27],38:26},{18:[2,49],30:[2,49],31:[2,49],32:[2,49],35:[2,49],37:[2,49],39:[2,49]},{18:[1,74]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],24:[2,22]},{18:[2,39],35:[2,39]},{18:[2,40],35:[2,40]},{18:[2,41],35:[2,41]},{18:[2,42],35:[2,42]},{18:[2,43],35:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],24:[2,18]}],
-defaultActions: {17:[2,1],25:[2,28],41:[2,26],61:[2,24]},
+table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],25:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],25:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],25:[1,16]},{17:23,18:[1,22],21:24,29:25,36:[1,28],38:[1,27],39:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],25:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],25:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],25:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],25:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],25:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],25:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],25:[2,15]},{17:31,21:24,29:25,36:[1,28],38:[1,27],39:26},{17:32,21:24,29:25,36:[1,28],38:[1,27],39:26},{17:33,21:24,29:25,36:[1,28],38:[1,27],39:26},{21:35,26:34,31:[1,36],32:[1,37],36:[1,28],39:26},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],25:[1,16]},{17:23,21:24,29:25,36:[1,28],38:[1,27],39:26},{5:[2,4],7:38,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],25:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],25:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],25:[2,23]},{18:[1,39]},{18:[2,27],21:44,24:[2,27],27:40,28:41,29:48,30:42,31:[1,45],32:[1,46],33:[1,47],34:43,35:49,36:[1,50],38:[1,27],39:26},{18:[2,28],24:[2,28]},{18:[2,48],24:[2,48],31:[2,48],32:[2,48],33:[2,48],36:[2,48],38:[2,48],40:[1,51]},{21:52,36:[1,28],39:26},{18:[2,50],24:[2,50],31:[2,50],32:[2,50],33:[2,50],36:[2,50],38:[2,50],40:[2,50]},{10:53,20:[1,54]},{10:55,20:[1,54]},{18:[1,56]},{18:[1,57]},{24:[1,58]},{18:[1,59],21:60,36:[1,28],39:26},{18:[2,44],36:[2,44]},{18:[2,45],36:[2,45]},{18:[2,46],36:[2,46]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],25:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],25:[2,17]},{18:[2,25],21:44,24:[2,25],28:61,29:48,30:62,31:[1,45],32:[1,46],33:[1,47],34:43,35:49,36:[1,50],38:[1,27],39:26},{18:[2,26],24:[2,26]},{18:[2,30],24:[2,30],31:[2,30],32:[2,30],33:[2,30],36:[2,30],38:[2,30]},{18:[2,36],24:[2,36],35:63,36:[1,64]},{18:[2,31],24:[2,31],31:[2,31],32:[2,31],33:[2,31],36:[2,31],38:[2,31]},{18:[2,32],24:[2,32],31:[2,32],32:[2,32],33:[2,32],36:[2,32],38:[2,32]},{18:[2,33],24:[2,33],31:[2,33],32:[2,33],33:[2,33],36:[2,33],38:[2,33]},{18:[2,34],24:[2,34],31:[2,34],32:[2,34],33:[2,34],36:[2,34],38:[2,34]},{18:[2,35],24:[2,35],31:[2,35],32:[2,35],33:[2,35],36:[2,35],38:[2,35]},{18:[2,38],24:[2,38],36:[2,38]},{18:[2,50],24:[2,50],31:[2,50],32:[2,50],33:[2,50],36:[2,50],37:[1,65],38:[2,50],40:[2,50]},{36:[1,66]},{18:[2,47],24:[2,47],31:[2,47],32:[2,47],33:[2,47],36:[2,47],38:[2,47]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],25:[2,10]},{21:67,36:[1,28],39:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],25:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],25:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],25:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],25:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],25:[2,21]},{18:[1,68]},{18:[2,24],24:[2,24]},{18:[2,29],24:[2,29],31:[2,29],32:[2,29],33:[2,29],36:[2,29],38:[2,29]},{18:[2,37],24:[2,37],36:[2,37]},{37:[1,65]},{21:69,29:73,31:[1,70],32:[1,71],33:[1,72],36:[1,28],38:[1,27],39:26},{18:[2,49],24:[2,49],31:[2,49],32:[2,49],33:[2,49],36:[2,49],38:[2,49],40:[2,49]},{18:[1,74]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],25:[2,22]},{18:[2,39],24:[2,39],36:[2,39]},{18:[2,40],24:[2,40],36:[2,40]},{18:[2,41],24:[2,41],36:[2,41]},{18:[2,42],24:[2,42],36:[2,42]},{18:[2,43],24:[2,43],36:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],25:[2,18]}],
+defaultActions: {17:[2,1]},
 parseError: function parseError(str, hash) {
     throw new Error(str);
 },
@@ -596,7 +599,7 @@ case 3:
 break;
 case 4: yy_.yytext = yy_.yytext.substr(0, yy_.yyleng-4); this.popState(); return 15; 
 break;
-case 5: return 24; 
+case 5: return 25; 
 break;
 case 6: return 16; 
 break;
@@ -608,7 +611,7 @@ case 9: return 19;
 break;
 case 10: return 23; 
 break;
-case 11: return 23; 
+case 11: return 22; 
 break;
 case 12: this.popState(); this.begin('com'); 
 break;
@@ -616,35 +619,35 @@ case 13: yy_.yytext = yy_.yytext.substr(3,yy_.yyleng-5); this.popState(); return
 break;
 case 14: return 22; 
 break;
-case 15: return 36; 
+case 15: return 37; 
 break;
-case 16: return 35; 
+case 16: return 36; 
 break;
-case 17: return 35; 
+case 17: return 36; 
 break;
-case 18: return 39; 
+case 18: return 40; 
 break;
 case 19: /*ignore whitespace*/ 
 break;
-case 20: this.popState(); return 18; 
+case 20: this.popState(); return 24; 
 break;
 case 21: this.popState(); return 18; 
 break;
-case 22: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\"/g,'"'); return 30; 
+case 22: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\"/g,'"'); return 31; 
 break;
-case 23: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\'/g,"'"); return 30; 
+case 23: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\'/g,"'"); return 31; 
 break;
-case 24: return 37; 
+case 24: return 38; 
 break;
-case 25: return 32; 
+case 25: return 33; 
 break;
-case 26: return 32; 
+case 26: return 33; 
 break;
-case 27: return 31; 
+case 27: return 32; 
 break;
-case 28: return 35; 
+case 28: return 36; 
 break;
-case 29: yy_.yytext = yy_.yytext.substr(1, yy_.yyleng-2); return 35; 
+case 29: yy_.yytext = yy_.yytext.substr(1, yy_.yyleng-2); return 36; 
 break;
 case 30: return 'INVALID'; 
 break;
diff --git a/release-notes.md b/release-notes.md
index 5a31418a9..53301626b 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -8,9 +8,14 @@
 - [#534](https://github.com/wycats/handlebars.js/issues/534) - Protect from object prototype modifications
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Allow ID or strings in partial names
+- [#437](https://github.com/wycats/handlebars.js/issues/437) - Require matching brace counts in escaped expressions
 - Add support for complex ids in @data references
 - Docs updates
 
+Compatibility notes:
+- The parser is now stricter on `{{{`, requiring that the end token be `}}}`. Templates that do not
+  follow this convention should add the additional brace value.
+
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
 
 ## v1.0.11 / 1.0.0-rc4 - May 13 2013
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 31c9b96a4..317841fdc 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -1571,6 +1571,15 @@ test('GH-534: Object prototype aliases', function() {
   delete Object.prototype[0xD834];
 });
 
+test('GH-437: Matching escaping', function() {
+  shouldThrow(function() {
+    CompilerContext.compile('{{{a}}');
+  }, Error);
+  shouldThrow(function() {
+    CompilerContext.compile('{{a}}}');
+  }, Error);
+});
+
 suite('Utils');
 
 test('escapeExpression', function() {
diff --git a/spec/tokenizer_spec.rb b/spec/tokenizer_spec.rb
index fc7a3acaa..0a7c3f9db 100644
--- a/spec/tokenizer_spec.rb
+++ b/spec/tokenizer_spec.rb
@@ -43,14 +43,15 @@ def tokenize(string)
 
   it "supports unescaping with &" do
     result = tokenize("{{&bar}}")
-    result.should match_tokens(%w(OPEN_UNESCAPED ID CLOSE))
+    result.should match_tokens(%w(OPEN ID CLOSE))
 
+    result[0].should be_token("OPEN", "{{&")
     result[1].should be_token("ID", "bar")
   end
 
   it "supports unescaping with {{{" do
     result = tokenize("{{{bar}}}")
-    result.should match_tokens(%w(OPEN_UNESCAPED ID CLOSE))
+    result.should match_tokens(%w(OPEN_UNESCAPED ID CLOSE_UNESCAPED))
 
     result[1].should be_token("ID", "bar")
   end
diff --git a/src/handlebars.l b/src/handlebars.l
index 2afd715bb..aa76eabd3 100644
--- a/src/handlebars.l
+++ b/src/handlebars.l
@@ -26,7 +26,7 @@
 <mu>"{{^"                        { return 'OPEN_INVERSE'; }
 <mu>"{{"\s*"else"                { return 'OPEN_INVERSE'; }
 <mu>"{{{"                        { return 'OPEN_UNESCAPED'; }
-<mu>"{{&"                        { return 'OPEN_UNESCAPED'; }
+<mu>"{{&"                        { return 'OPEN'; }
 <mu>"{{!--"                      { this.popState(); this.begin('com'); }
 <mu>"{{!"[\s\S]*?"}}"            { yytext = yytext.substr(3,yyleng-5); this.popState(); return 'COMMENT'; }
 <mu>"{{"                         { return 'OPEN'; }
@@ -36,7 +36,7 @@
 <mu>".."                         { return 'ID'; }
 <mu>[\/.]                        { return 'SEP'; }
 <mu>\s+                          { /*ignore whitespace*/ }
-<mu>"}}}"                        { this.popState(); return 'CLOSE'; }
+<mu>"}}}"                        { this.popState(); return 'CLOSE_UNESCAPED'; }
 <mu>"}}"                         { this.popState(); return 'CLOSE'; }
 <mu>'"'("\\"["]|[^"])*'"'        { yytext = yytext.substr(1,yyleng-2).replace(/\\"/g,'"'); return 'STRING'; }
 <mu>"'"("\\"[']|[^'])*"'"        { yytext = yytext.substr(1,yyleng-2).replace(/\\'/g,"'"); return 'STRING'; }
diff --git a/src/handlebars.yy b/src/handlebars.yy
index e521c2e5e..56b3b7040 100644
--- a/src/handlebars.yy
+++ b/src/handlebars.yy
@@ -42,8 +42,11 @@ closeBlock
   ;
 
 mustache
-  : OPEN inMustache CLOSE { $$ = new yy.MustacheNode($2[0], $2[1]); }
-  | OPEN_UNESCAPED inMustache CLOSE { $$ = new yy.MustacheNode($2[0], $2[1], true); }
+  : OPEN inMustache CLOSE {
+    // Parsing out the '&' escape token at this level saves ~500 bytes after min due to the removal of one parser node.
+    $$ = new yy.MustacheNode($2[0], $2[1], $1[2] === '&');
+  }
+  | OPEN_UNESCAPED inMustache CLOSE_UNESCAPED { $$ = new yy.MustacheNode($2[0], $2[1], true); }
   ;
 
 

From 17659b972070ee081987c410f1dbbf569a41f2e1 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Wed, 29 May 2013 15:31:08 -0400
Subject: [PATCH 28/34] Ignore minimized files from git

---
 .gitignore | 1 +
 1 file changed, 1 insertion(+)

diff --git a/.gitignore b/.gitignore
index 4b5a11844..6e326b1ba 100644
--- a/.gitignore
+++ b/.gitignore
@@ -2,6 +2,7 @@ vendor
 .rvmrc
 .DS_Store
 lib/handlebars/compiler/parser.js
+dist/*.min.js
 node_modules
 *.sublime-project
 *.sublime-workspace

From d7b345b2da5436cd3199d9b9fcf51f0af49da77c Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Thu, 30 May 2013 15:46:00 -0400
Subject: [PATCH 29/34] Allow execution of helpers on the context

Fixes #285
---
 dist/handlebars.js                  | 3 ++-
 lib/handlebars/compiler/compiler.js | 3 ++-
 spec/qunit_spec.js                  | 8 ++++++++
 3 files changed, 12 insertions(+), 2 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index a5f0bf7e6..418b80409 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -1743,8 +1743,9 @@ JavaScriptCompiler.prototype = {
     this.context.aliases.helperMissing = 'helpers.helperMissing';
 
     var helper = this.lastHelper = this.setupHelper(paramSize, name, true);
+    var nonHelper = this.nameLookup('depth' + this.lastContext, name, 'context');
 
-    this.push(helper.name);
+    this.push(helper.name + ' || ' + nonHelper);
     this.replaceStack(function(name) {
       return name + ' ? ' + name + '.call(' +
           helper.callParams + ") " + ": helperMissing.call(" +
diff --git a/lib/handlebars/compiler/compiler.js b/lib/handlebars/compiler/compiler.js
index 7777456ce..c003ce617 100644
--- a/lib/handlebars/compiler/compiler.js
+++ b/lib/handlebars/compiler/compiler.js
@@ -866,8 +866,9 @@ JavaScriptCompiler.prototype = {
     this.context.aliases.helperMissing = 'helpers.helperMissing';
 
     var helper = this.lastHelper = this.setupHelper(paramSize, name, true);
+    var nonHelper = this.nameLookup('depth' + this.lastContext, name, 'context');
 
-    this.push(helper.name);
+    this.push(helper.name + ' || ' + nonHelper);
     this.replaceStack(function(name) {
       return name + ' ? ' + name + '.call(' +
           helper.callParams + ") " + ": helperMissing.call(" +
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 317841fdc..14dbee7eb 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -165,6 +165,14 @@ test("functions", function() {
                   "functions are bound to the context");
 });
 
+test("functions with context argument", function() {
+  shouldCompileTo("{{awesome frank}}",
+      {awesome: function(context) { return context; },
+        frank: "Frank"},
+      "Frank", "functions are called with context arguments");
+});
+
+
 test("paths with hyphens", function() {
   shouldCompileTo("{{foo-bar}}", {"foo-bar": "baz"}, "baz", "Paths can contain hyphens (-)");
   shouldCompileTo("{{foo.foo-bar}}", {foo: {"foo-bar": "baz"}}, "baz", "Paths can contain hyphens (-)");

From 20242095197c998d89d1bc2d08895422c73f36d4 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Thu, 30 May 2013 15:57:15 -0400
Subject: [PATCH 30/34] Move upgrade notes to release-notes

---
 README.markdown  | 16 +---------------
 release-notes.md | 18 ++++++++++++++++++
 2 files changed, 19 insertions(+), 15 deletions(-)

diff --git a/README.markdown b/README.markdown
index 9f7a12223..f775815e2 100644
--- a/README.markdown
+++ b/README.markdown
@@ -329,21 +329,7 @@ in the `dist` directory.
 Upgrading
 ---------
 
-When upgrading from the Handlebars 0.9 series, be aware that the
-signature for passing custom helpers or partials to templates has
-changed.
-
-Instead of:
-
-```js
-template(context, helpers, partials, [data])
-```
-
-Use:
-
-```js
-template(context, {helpers: helpers, partials: partials, data: data})
-```
+See [release-notes.md](https://github.com/wycats/handlebars.js/blob/master/release-notes.md) for upgrade notes.
 
 Known Issues
 ------------
diff --git a/release-notes.md b/release-notes.md
index 53301626b..bb4456086 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -67,3 +67,21 @@ Compatibility notes:
 - Package browser dist in npm package
 
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.8...1.0.0-rc.3)
+
+## Prior Versions
+
+When upgrading from the Handlebars 0.9 series, be aware that the
+signature for passing custom helpers or partials to templates has
+changed.
+
+Instead of:
+
+```js
+template(context, helpers, partials, [data])
+```
+
+Use:
+
+```js
+template(context, {helpers: helpers, partials: partials, data: data})
+```

From ef062adcc24ea2c1c617cbe54de6d8982d739a40 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Fri, 31 May 2013 13:11:22 -0400
Subject: [PATCH 31/34] Move external AST tests into qunit spec

---
 spec/parser_spec.rb | 11 -----------
 spec/qunit_spec.js  |  4 ++++
 2 files changed, 4 insertions(+), 11 deletions(-)

diff --git a/spec/parser_spec.rb b/spec/parser_spec.rb
index 3ee011778..b25e88938 100644
--- a/spec/parser_spec.rb
+++ b/spec/parser_spec.rb
@@ -417,21 +417,10 @@ def path(*parts)
   end
 
   context "externally compiled AST" do
-
     it "can pass through an already-compiled AST" do
       ast_for(@context.eval('new Handlebars.AST.ProgramNode([ new Handlebars.AST.ContentNode("Hello")]);')).should == root do
         content "Hello"
       end
     end
-
-    it "can pass through an already-compiled AST via compile/precompile" do
-      @context = Handlebars::Spec::CONTEXT
-
-      code = 'Handlebars.compile(new Handlebars.AST.ProgramNode([ new Handlebars.AST.ContentNode("Hello")]))();'
-      @context.eval(code).should == "Hello"
-
-      code = @context.eval 'Handlebars.precompile(new Handlebars.AST.ProgramNode([ new Handlebars.AST.ContentNode("Hello")]))'
-      @context.eval("(#{code})(this)").should == "Hello"
-    end
   end
 end
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index 14dbee7eb..ec58570f1 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -1538,6 +1538,10 @@ test("Passing falsy values to Handlebars.compile throws an error", function() {
   }, "You must pass a string or Handlebars AST to Handlebars.precompile. You passed null");
 });
 
+test("can pass through an already-compiled AST via compile/precompile", function() {
+  equal(Handlebars.compile(new Handlebars.AST.ProgramNode([ new Handlebars.AST.ContentNode("Hello")]))(), 'Hello')
+});
+
 test('GH-408: Multiple loops fail', function() {
   var context = [
     { name: "John Doe", location: { city: "Chicago" } },

From 16fd601a5caab026f26fa8adb63e90e22d9f5d77 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Fri, 31 May 2013 13:12:06 -0400
Subject: [PATCH 32/34] Merge global and passed helpers and partials

---
 dist/handlebars.js                  | 20 ++++++++++++++++----
 dist/handlebars.runtime.js          | 15 +++++++++++++--
 lib/handlebars/base.js              |  5 +++--
 lib/handlebars/compiler/compiler.js |  5 +++--
 lib/handlebars/runtime.js           | 10 ++++++++++
 release-notes.md                    |  2 ++
 spec/qunit_spec.js                  | 28 ++++++++++++++++++++--------
 7 files changed, 67 insertions(+), 18 deletions(-)

diff --git a/dist/handlebars.js b/dist/handlebars.js
index 418b80409..e2c394f49 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -30,12 +30,13 @@ var Handlebars = {};
 // lib/handlebars/base.js
 
 Handlebars.VERSION = "1.0.0-rc.4";
-Handlebars.COMPILER_REVISION = 3;
+Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
   1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
   2: '== 1.0.0-rc.3',
-  3: '>= 1.0.0-rc.4'
+  3: '== 1.0.0-rc.4',
+  4: '>= 1.0.0'
 };
 
 Handlebars.helpers  = {};
@@ -1385,8 +1386,9 @@ JavaScriptCompiler.prototype = {
 
     if (!this.isChild) {
       var namespace = this.namespace;
-      var copies = "helpers = helpers || " + namespace + ".helpers;";
-      if (this.environment.usePartial) { copies = copies + " partials = partials || " + namespace + ".partials;"; }
+
+      var copies = "helpers = this.merge(helpers, " + namespace + ".helpers);";
+      if (this.environment.usePartial) { copies = copies + " partials = this.merge(partials, " + namespace + ".partials);"; }
       if (this.options.data) { copies = copies + " data = data || {};"; }
       out.push(copies);
     } else {
@@ -2190,6 +2192,16 @@ Handlebars.VM = {
         }
         return programWrapper;
       },
+      merge: function(param, common) {
+        var ret = param || common;
+
+        if (param && common) {
+          ret = {};
+          Handlebars.Utils.extend(ret, common);
+          Handlebars.Utils.extend(ret, param);
+        }
+        return ret;
+      },
       programWithDepth: Handlebars.VM.programWithDepth,
       noop: Handlebars.VM.noop,
       compilerInfo: null
diff --git a/dist/handlebars.runtime.js b/dist/handlebars.runtime.js
index d11c79d54..7e88761ab 100644
--- a/dist/handlebars.runtime.js
+++ b/dist/handlebars.runtime.js
@@ -30,12 +30,13 @@ var Handlebars = {};
 // lib/handlebars/base.js
 
 Handlebars.VERSION = "1.0.0-rc.4";
-Handlebars.COMPILER_REVISION = 3;
+Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
   1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
   2: '== 1.0.0-rc.3',
-  3: '>= 1.0.0-rc.4'
+  3: '== 1.0.0-rc.4',
+  4: '>= 1.0.0'
 };
 
 Handlebars.helpers  = {};
@@ -275,6 +276,16 @@ Handlebars.VM = {
         }
         return programWrapper;
       },
+      merge: function(param, common) {
+        var ret = param || common;
+
+        if (param && common) {
+          ret = {};
+          Handlebars.Utils.extend(ret, common);
+          Handlebars.Utils.extend(ret, param);
+        }
+        return ret;
+      },
       programWithDepth: Handlebars.VM.programWithDepth,
       noop: Handlebars.VM.noop,
       compilerInfo: null
diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index ad2d9ec2c..e1cd712d2 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -7,12 +7,13 @@ var Handlebars = {};
 // BEGIN(BROWSER)
 
 Handlebars.VERSION = "1.0.0-rc.4";
-Handlebars.COMPILER_REVISION = 3;
+Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
   1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
   2: '== 1.0.0-rc.3',
-  3: '>= 1.0.0-rc.4'
+  3: '== 1.0.0-rc.4',
+  4: '>= 1.0.0'
 };
 
 Handlebars.helpers  = {};
diff --git a/lib/handlebars/compiler/compiler.js b/lib/handlebars/compiler/compiler.js
index c003ce617..8bb1fc521 100644
--- a/lib/handlebars/compiler/compiler.js
+++ b/lib/handlebars/compiler/compiler.js
@@ -508,8 +508,9 @@ JavaScriptCompiler.prototype = {
 
     if (!this.isChild) {
       var namespace = this.namespace;
-      var copies = "helpers = helpers || " + namespace + ".helpers;";
-      if (this.environment.usePartial) { copies = copies + " partials = partials || " + namespace + ".partials;"; }
+
+      var copies = "helpers = this.merge(helpers, " + namespace + ".helpers);";
+      if (this.environment.usePartial) { copies = copies + " partials = this.merge(partials, " + namespace + ".partials);"; }
       if (this.options.data) { copies = copies + " data = data || {};"; }
       out.push(copies);
     } else {
diff --git a/lib/handlebars/runtime.js b/lib/handlebars/runtime.js
index d99019d9e..2e845c4cc 100644
--- a/lib/handlebars/runtime.js
+++ b/lib/handlebars/runtime.js
@@ -18,6 +18,16 @@ Handlebars.VM = {
         }
         return programWrapper;
       },
+      merge: function(param, common) {
+        var ret = param || common;
+
+        if (param && common) {
+          ret = {};
+          Handlebars.Utils.extend(ret, common);
+          Handlebars.Utils.extend(ret, param);
+        }
+        return ret;
+      },
       programWithDepth: Handlebars.VM.programWithDepth,
       noop: Handlebars.VM.noop,
       compilerInfo: null
diff --git a/release-notes.md b/release-notes.md
index bb4456086..477fc2261 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -9,12 +9,14 @@
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Fix partials with . name ([@jamesgorrie](https://github.com/jamesgorrie))
 - [#519](https://github.com/wycats/handlebars.js/issues/519) - Allow ID or strings in partial names
 - [#437](https://github.com/wycats/handlebars.js/issues/437) - Require matching brace counts in escaped expressions
+- Merge passed partials and helpers with global namespace values
 - Add support for complex ids in @data references
 - Docs updates
 
 Compatibility notes:
 - The parser is now stricter on `{{{`, requiring that the end token be `}}}`. Templates that do not
   follow this convention should add the additional brace value.
+- Code that relies on global the namespace being muted when custom helpers or partials are passed will need to explicitly pass an `undefined` value for any helpers that should not be available.
 
 [Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
 
diff --git a/spec/qunit_spec.js b/spec/qunit_spec.js
index ec58570f1..5d52e444b 100644
--- a/spec/qunit_spec.js
+++ b/spec/qunit_spec.js
@@ -42,14 +42,6 @@ function shouldCompileToWithPartials(string, hashOrArray, partials, expected, me
 function compileWithPartials(string, hashOrArray, partials) {
   var template = CompilerContext[partials ? 'compileWithPartial' : 'compile'](string), ary;
   if(Object.prototype.toString.call(hashOrArray) === "[object Array]") {
-    var helpers = hashOrArray[1];
-
-    if(helpers) {
-      for(var prop in Handlebars.helpers) {
-        helpers[prop] = helpers[prop] || Handlebars.helpers[prop];
-      }
-    }
-
     ary = [];
     ary.push(hashOrArray[0]);
     ary.push({ helpers: hashOrArray[1], partials: hashOrArray[2] });
@@ -500,6 +492,17 @@ test("the helpers hash is available is nested contexts", function() {
                 "helpers hash is available in nested contexts.");
 });
 
+test("the helper hash should augment the global hash", function() {
+  Handlebars.registerHelper('test_helper', function() { return 'found it!'; });
+
+  shouldCompileTo(
+    "{{test_helper}} {{#if cruel}}Goodbye {{cruel}} {{world}}!{{/if}}", [
+      {cruel: "cruel"},
+      {world: function() { return "world!"; }}
+    ],
+    "found it! Goodbye cruel world!!");
+});
+
 test("Multiple global helper registration", function() {
   var helpers = Handlebars.helpers;
   try {
@@ -592,6 +595,15 @@ test("Partials with slash and point paths", function() {
   shouldCompileToWithPartials(string, [hash, {}, {'shared/dude.thing':dude}], true, "Dudes: Jeepers", "Partials can use literal with points in paths");
 });
 
+test("Global Partials", function() {
+  Handlebars.registerPartial('global_test', '{{another_dude}}');
+
+  var string = "Dudes: {{> shared/dude}} {{> global_test}}";
+  var dude = "{{name}}";
+  var hash = {name:"Jeepers", another_dude:"Creepers"};
+  shouldCompileToWithPartials(string, [hash, {}, {'shared/dude':dude}], true, "Dudes: Jeepers Creepers", "Partials can use globals or passed");
+});
+
 test("Multiple partial registration", function() {
   Handlebars.registerPartial({
     'shared/dude': '{{name}}',

From f63b226b7659d73b71bb7dd58c78ab4df1498633 Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Fri, 31 May 2013 14:16:19 -0400
Subject: [PATCH 33/34] Update to 1.0.0 final

---
 bower.json                 | 2 +-
 dist/handlebars.js         | 2 +-
 dist/handlebars.runtime.js | 2 +-
 handlebars.js.nuspec       | 2 +-
 lib/handlebars/base.js     | 2 +-
 release-notes.md           | 6 +++++-
 6 files changed, 10 insertions(+), 6 deletions(-)

diff --git a/bower.json b/bower.json
index a5eb00b98..4b86a80a4 100644
--- a/bower.json
+++ b/bower.json
@@ -1,6 +1,6 @@
 {
   "name": "handlebars.js",
-  "version": "1.0.0-rc.4",
+  "version": "1.0.0",
   "main": "dist/handlebars.js",
   "ignore": [
     "node_modules",
diff --git a/dist/handlebars.js b/dist/handlebars.js
index e2c394f49..c70f09d1d 100644
--- a/dist/handlebars.js
+++ b/dist/handlebars.js
@@ -29,7 +29,7 @@ var Handlebars = {};
 ;
 // lib/handlebars/base.js
 
-Handlebars.VERSION = "1.0.0-rc.4";
+Handlebars.VERSION = "1.0.0";
 Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
diff --git a/dist/handlebars.runtime.js b/dist/handlebars.runtime.js
index 7e88761ab..b9f4e4920 100644
--- a/dist/handlebars.runtime.js
+++ b/dist/handlebars.runtime.js
@@ -29,7 +29,7 @@ var Handlebars = {};
 ;
 // lib/handlebars/base.js
 
-Handlebars.VERSION = "1.0.0-rc.4";
+Handlebars.VERSION = "1.0.0";
 Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
diff --git a/handlebars.js.nuspec b/handlebars.js.nuspec
index 9a16cc09d..4515f2ebd 100644
--- a/handlebars.js.nuspec
+++ b/handlebars.js.nuspec
@@ -2,7 +2,7 @@
 <package>
 	<metadata>
 		<id>handlebars.js</id>
-		<version>1.0.0-rc.4</version>
+		<version>1.0.0</version>
 		<authors>handlebars.js Authors</authors>
 		<licenseUrl>https://github.com/wycats/handlebars.js/blob/master/LICENSE</licenseUrl>
 		<projectUrl>https://github.com/wycats/handlebars.js/</projectUrl>
diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index e1cd712d2..44a369c5e 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -6,7 +6,7 @@ var Handlebars = {};
 
 // BEGIN(BROWSER)
 
-Handlebars.VERSION = "1.0.0-rc.4";
+Handlebars.VERSION = "1.0.0";
 Handlebars.COMPILER_REVISION = 4;
 
 Handlebars.REVISION_CHANGES = {
diff --git a/release-notes.md b/release-notes.md
index 477fc2261..8e6761944 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -1,6 +1,10 @@
 # Release Notes
 
 ## Development
+[Commits](https://github.com/wycats/handlebars.js/compare/v1.0.12...master)
+
+## v1.0.12 / 1.0.0 - May 31 2013
+
 - [#515](https://github.com/wycats/handlebars.js/issues/515) - Add node require extensions support ([@jjclark1982](https://github.com/jjclark1982))
 - [#517](https://github.com/wycats/handlebars.js/issues/517) - Fix amd precompiler output with directories ([@blessenm](https://github.com/blessenm))
 - [#433](https://github.com/wycats/handlebars.js/issues/433) - Add support for unicode ids
@@ -18,7 +22,7 @@ Compatibility notes:
   follow this convention should add the additional brace value.
 - Code that relies on global the namespace being muted when custom helpers or partials are passed will need to explicitly pass an `undefined` value for any helpers that should not be available.
 
-[Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...master)
+[Commits](https://github.com/wycats/handlebars.js/compare/v1.0.11...v1.0.12)
 
 ## v1.0.11 / 1.0.0-rc4 - May 13 2013
 

From 2a073e0993b40b81fbef82f681bb1dd171f2233b Mon Sep 17 00:00:00 2001
From: kpdecker <kpdecker@gmail.com>
Date: Fri, 31 May 2013 14:16:53 -0400
Subject: [PATCH 34/34] 1.0.12

---
 package.json | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/package.json b/package.json
index af8db6855..b469398f2 100644
--- a/package.json
+++ b/package.json
@@ -1,7 +1,7 @@
 {
   "name": "handlebars",
   "description": "Extension of the Mustache logicless template language",
-  "version": "1.0.11",
+  "version": "1.0.12",
   "homepage": "http://www.handlebarsjs.com/",
   "keywords": [
     "handlebars mustache template html"
